import logging
import time
from typing import Generator, List, Optional, Tuple, Union

import grpc
from osmosis_proto.proto.cosmos.auth.v1beta1 import auth_pb2 as auth_type
from osmosis_proto.proto.cosmos.auth.v1beta1 import query_pb2 as auth_query
from osmosis_proto.proto.cosmos.auth.v1beta1 import query_pb2_grpc as auth_query_grpc
from osmosis_proto.proto.cosmos.authz.v1beta1 import query_pb2 as authz_query
from osmosis_proto.proto.cosmos.authz.v1beta1 import query_pb2_grpc as authz_query_grpc
from osmosis_proto.proto.cosmos.bank.v1beta1 import query_pb2 as bank_query
from osmosis_proto.proto.cosmos.bank.v1beta1 import query_pb2_grpc as bank_query_grpc
from osmosis_proto.proto.cosmos.base.abci.v1beta1 import abci_pb2 as abci_type
from osmosis_proto.proto.cosmos.base.tendermint.v1beta1 import (
    query_pb2 as tendermint_query,
)
from osmosis_proto.proto.cosmos.base.tendermint.v1beta1 import (
    query_pb2_grpc as tendermint_query_grpc,
)
from osmosis_proto.proto.cosmos.tx.v1beta1 import service_pb2 as tx_service
from osmosis_proto.proto.cosmos.tx.v1beta1 import service_pb2_grpc as tx_service_grpc
from packaging import version

from osmosispy import pytypes

DEFAULT_TIMEOUTHEIGHT = 20  # blocks
GITHUB_COMMIT_HASH_LEN = 40


class GrpcClient:
    def __init__(
        self,
        network: pytypes.Network,
        insecure=False,
        credentials: grpc.ChannelCredentials = None,
    ):
        """
        _summary_

        Args:
            network (Network): The network object
            insecure (bool, optional): Wether the network should use ssl or not. Defaults to False.
            credentials (grpc.ChannelCredentials, optional): Ssl creds. Defaults to None.
        """

        # load root CA cert
        if not insecure:
            credentials = grpc.ssl_channel_credentials()

        # chain stubs
        grpc_endpoint = network.grpc_endpoint.lstrip("tcp://")
        self.chain_channel = (
            grpc.insecure_channel(grpc_endpoint)
            if insecure
            else grpc.secure_channel(grpc_endpoint, credentials)
        )

        self.stubCosmosTendermint = tendermint_query_grpc.ServiceStub(
            self.chain_channel
        )
        self.stubAuth = auth_query_grpc.QueryStub(self.chain_channel)
        self.stubAuthz = authz_query_grpc.QueryStub(self.chain_channel)
        self.stubBank = bank_query_grpc.QueryStub(self.chain_channel)
        self.stubTx = tx_service_grpc.ServiceStub(self.chain_channel)

        self.timeout_height = 1

    @staticmethod
    def assert_compatible_versions(osmosis_proto_version, chain_osmo_version):
        """
        Assert that this version of the python sdk is compatible with the chain.
        If you run the chain from a non tagged release, the version query will be returning something like
        master-6a315bab3db46f5fa1158199acc166ed2d192c2f. Otherwise, it should be for example `v0.14.0`.

        If the chain is running a custom non tagged release, you are free to use the python sdk at your own risk.
        """
        if osmosis_proto_version[0] == "v":
            osmosis_proto_version = osmosis_proto_version[1:]
        if chain_osmo_version[0] == "v":
            chain_osmo_version = chain_osmo_version[1:]

        if len(chain_osmo_version) >= GITHUB_COMMIT_HASH_LEN:
            logger = logging.getLogger("client-logger")
            logger.warning(
                f"The chain is running a custom release from branch/commit {chain_osmo_version}. "
                "We bypass the compatibility assertion"
            )
            logger.warning(
                f"The chain is running a custom release from branch/commit {chain_osmo_version}"
            )
        else:
            error_string = (
                f"Version error, Python sdk runs with osmosis protobuf version {osmosis_proto_version}, but the "
                f"remote chain is running with version {chain_osmo_version}"
            )

            assert (
                version.parse(osmosis_proto_version).major
                >= version.parse(chain_osmo_version).major
            ) and (
                version.parse(osmosis_proto_version).minor
                >= version.parse(chain_osmo_version).minor
            ), error_string

    def close_chain_channel(self):
        self.chain_channel.close()

    def sync_timeout_height(self):
        block = self.get_latest_block()
        self.timeout_height = block.block.header.height + DEFAULT_TIMEOUTHEIGHT

    def wait_for_next_block(self):
        """
        Wait for a block to be written
        """
        current_block = self.get_latest_block().block.header.height
        while self.get_latest_block().block.header.height < current_block + 1:
            time.sleep(0.5)

    def get_block_by_height(
        self, height: int
    ) -> tendermint_query.GetBlockByHeightResponse:
        """
        Returns the block specified by height

        Args:
            height: the height of the block

        Returns:
            tendermint_query.GetBlockByHeightResponse: the block info

        """
        req = tendermint_query.GetBlockByHeightRequest(height=height)
        return self.stubCosmosTendermint.GetBlockByHeight(req)

    def get_blocks_by_height(
        self, start_height: int, end_height: int = None
    ) -> Generator[tendermint_query.GetBlockByHeightResponse, None, None]:
        """
        Iterate through all the blocks in the chain and yield the output of the block one by one.
        If no end_height is specified, iterate until the current latest block is reached.

        Args:
            start_height (int): The starting block height
            end_height (int, optional): The last block height to query. Defaults to None.

        Yields:
            Generator[tendermint_query.GetBlockByHeightResponse, None, None]
        """
        if end_height is None:
            height = start_height
            while True:
                try:
                    yield self.get_block_by_height(height)
                    height += 1
                except:
                    break
        else:
            for height in range(start_height, end_height):
                yield self.get_block_by_height(height)

    # default client methods
    def get_latest_block(self) -> tendermint_query.GetLatestBlockResponse:
        """
        Returns the last block

        Returns:
            tendermint_query.GetLatestBlockResponse: the last block information

        """
        req = tendermint_query.GetLatestBlockRequest()
        return self.stubCosmosTendermint.GetLatestBlock(req)

    def get_version(self) -> str:
        """
        Returns the application version

        Returns:
            str: the version of the app

        """
        req = tendermint_query.GetNodeInfoRequest()
        version = self.stubCosmosTendermint.GetNodeInfo(
            req).application_version.version

        if version[0] != "v":
            version = "v" + str(version)

        return version

    def get_latest_block_height(self) -> int:
        """
        Returns the latest block height

        Returns:
            int: the last block height

        """
        return self.get_latest_block().block.header.height

    def get_account(self, address: str) -> Optional[auth_type.BaseAccount]:
        """
        Returns the account info from address

        Args:
            address: the address of the account

        Returns:
            Optional[auth_type.BaseAccount]: the account information, none if not found

        """
        try:
            account_any = self.stubAuth.Account(
                auth_query.QueryAccountRequest(address=address)
            ).account
            account = auth_type.BaseAccount()
            if account_any.Is(account.DESCRIPTOR):
                account_any.Unpack(account)
                return account
        except:
            return None

    def get_request_id_by_tx_hash(self, tx_hash: bytes) -> List[int]:
        tx = self.stubTx.GetTx(tx_service.GetTxRequest(hash=tx_hash))
        request_ids = []
        for tx in tx.tx_response.logs:
            request_event = [
                event
                for event in tx.events
                if event.type == "request" or event.type == "report"
            ]
            if len(request_event) == 1:
                attrs = request_event[0].attributes
                attr_id = [attr for attr in attrs if attr.key == "id"]
                if len(attr_id) == 1:
                    request_id = attr_id[0].value
                    request_ids.append(int(request_id))
        if len(request_ids) == 0:
            raise KeyError("Request Id is not found")
        return request_ids

    def simulate_tx(
        self, tx_byte: bytes
    ) -> Tuple[Union[abci_type.SimulationResponse, grpc.RpcError], bool]:
        try:
            req = tx_service.SimulateRequest(tx_bytes=tx_byte)
            return self.stubTx.Simulate.__call__(req), True
        except grpc.RpcError as err:
            return err._state.details, False

    def send_tx_sync_mode(self, tx_byte: bytes) -> abci_type.TxResponse:
        """
        Sends a transaction in sync mode

        Args:
            tx_byte: the tx in bytes

        Returns:
            abci_type.TxResponse: the tx response

        """
        req = tx_service.BroadcastTxRequest(
            tx_bytes=tx_byte, mode=tx_service.BroadcastMode.BROADCAST_MODE_SYNC
        )
        result = self.stubTx.BroadcastTx.__call__(req)
        return result.tx_response

    def send_tx_async_mode(self, tx_byte: bytes) -> abci_type.TxResponse:
        """
        Sends a transaction in async mode

        Args:
            tx_byte: the tx in bytes

        Returns:
            abci_type.TxResponse: the tx response

        """
        req = tx_service.BroadcastTxRequest(
            tx_bytes=tx_byte, mode=tx_service.BroadcastMode.BROADCAST_MODE_ASYNC
        )
        result = self.stubTx.BroadcastTx.__call__(req)
        return result.tx_response

    def send_tx_block_mode(self, tx_byte: bytes) -> abci_type.TxResponse:
        """
        Sends a transaction in block mode

        Args:
            tx_byte: the tx in bytes

        Returns:
            abci_type.TxResponse: the tx response

        """
        req = tx_service.BroadcastTxRequest(
            tx_bytes=tx_byte, mode=tx_service.BroadcastMode.BROADCAST_MODE_BLOCK
        )
        result = self.stubTx.BroadcastTx.__call__(req)
        return result.tx_response



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import osmosis.tokenfactory.v1beta1.authorityMetadata_pb2
import osmosis.tokenfactory.v1beta1.params_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class QueryParamsRequest(google.protobuf.message.Message):
    """QueryParamsRequest is the request type for the Query/Params RPC method."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___QueryParamsRequest = QueryParamsRequest

@typing_extensions.final
class QueryParamsResponse(google.protobuf.message.Message):
    """QueryParamsResponse is the response type for the Query/Params RPC method."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PARAMS_FIELD_NUMBER: builtins.int
    @property
    def params(self) -> osmosis.tokenfactory.v1beta1.params_pb2.Params:
        """params defines the parameters of the module."""
    def __init__(
        self,
        *,
        params: osmosis.tokenfactory.v1beta1.params_pb2.Params | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["params", b"params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["params", b"params"]) -> None: ...

global___QueryParamsResponse = QueryParamsResponse

@typing_extensions.final
class QueryDenomAuthorityMetadataRequest(google.protobuf.message.Message):
    """QueryDenomAuthorityMetadataRequest defines the request structure for the
    DenomAuthorityMetadata gRPC query.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DENOM_FIELD_NUMBER: builtins.int
    denom: builtins.str
    def __init__(
        self,
        *,
        denom: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["denom", b"denom"]) -> None: ...

global___QueryDenomAuthorityMetadataRequest = QueryDenomAuthorityMetadataRequest

@typing_extensions.final
class QueryDenomAuthorityMetadataResponse(google.protobuf.message.Message):
    """QueryDenomAuthorityMetadataResponse defines the response structure for the
    DenomAuthorityMetadata gRPC query.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AUTHORITY_METADATA_FIELD_NUMBER: builtins.int
    @property
    def authority_metadata(self) -> osmosis.tokenfactory.v1beta1.authorityMetadata_pb2.DenomAuthorityMetadata: ...
    def __init__(
        self,
        *,
        authority_metadata: osmosis.tokenfactory.v1beta1.authorityMetadata_pb2.DenomAuthorityMetadata | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["authority_metadata", b"authority_metadata"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["authority_metadata", b"authority_metadata"]) -> None: ...

global___QueryDenomAuthorityMetadataResponse = QueryDenomAuthorityMetadataResponse

@typing_extensions.final
class QueryDenomsFromCreatorRequest(google.protobuf.message.Message):
    """QueryDenomsFromCreatorRequest defines the request structure for the
    DenomsFromCreator gRPC query.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CREATOR_FIELD_NUMBER: builtins.int
    creator: builtins.str
    def __init__(
        self,
        *,
        creator: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["creator", b"creator"]) -> None: ...

global___QueryDenomsFromCreatorRequest = QueryDenomsFromCreatorRequest

@typing_extensions.final
class QueryDenomsFromCreatorResponse(google.protobuf.message.Message):
    """QueryDenomsFromCreatorRequest defines the response structure for the
    DenomsFromCreator gRPC query.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DENOMS_FIELD_NUMBER: builtins.int
    @property
    def denoms(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        denoms: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["denoms", b"denoms"]) -> None: ...

global___QueryDenomsFromCreatorResponse = QueryDenomsFromCreatorResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class GenesisState(google.protobuf.message.Message):
    """GenesisState defines the raw genesis transaction in JSON."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    GEN_TXS_FIELD_NUMBER: builtins.int
    @property
    def gen_txs(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bytes]:
        """gen_txs defines the genesis transactions."""
    def __init__(
        self,
        *,
        gen_txs: collections.abc.Iterable[builtins.bytes] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["gen_txs", b"gen_txs"]) -> None: ...

global___GenesisState = GenesisState



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.bank.v1beta1.bank_pb2
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class MsgCreateDenom(google.protobuf.message.Message):
    """MsgCreateDenom defines the message structure for the CreateDenom gRPC service
    method. It allows an account to create a new denom. It requires a sender
    address and a sub denomination. The (sender_address, sub_denomination) tuple
    must be unique and cannot be re-used.

    The resulting denom created is defined as
    <factory/{creatorAddress}/{subdenom}>. The resulting denom's admin is
    originally set to be the creator, but this can be changed later. The token
    denom does not indicate the current admin.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SENDER_FIELD_NUMBER: builtins.int
    SUBDENOM_FIELD_NUMBER: builtins.int
    sender: builtins.str
    subdenom: builtins.str
    """subdenom can be up to 44 "alphanumeric" characters long."""
    def __init__(
        self,
        *,
        sender: builtins.str = ...,
        subdenom: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["sender", b"sender", "subdenom", b"subdenom"]) -> None: ...

global___MsgCreateDenom = MsgCreateDenom

@typing_extensions.final
class MsgCreateDenomResponse(google.protobuf.message.Message):
    """MsgCreateDenomResponse is the return value of MsgCreateDenom
    It returns the full string of the newly created denom
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NEW_TOKEN_DENOM_FIELD_NUMBER: builtins.int
    new_token_denom: builtins.str
    def __init__(
        self,
        *,
        new_token_denom: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["new_token_denom", b"new_token_denom"]) -> None: ...

global___MsgCreateDenomResponse = MsgCreateDenomResponse

@typing_extensions.final
class MsgMint(google.protobuf.message.Message):
    """MsgMint is the sdk.Msg type for allowing an admin account to mint
    more of a token.  For now, we only support minting to the sender account
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SENDER_FIELD_NUMBER: builtins.int
    AMOUNT_FIELD_NUMBER: builtins.int
    sender: builtins.str
    @property
    def amount(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    def __init__(
        self,
        *,
        sender: builtins.str = ...,
        amount: cosmos.base.v1beta1.coin_pb2.Coin | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["amount", b"amount"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["amount", b"amount", "sender", b"sender"]) -> None: ...

global___MsgMint = MsgMint

@typing_extensions.final
class MsgMintResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___MsgMintResponse = MsgMintResponse

@typing_extensions.final
class MsgBurn(google.protobuf.message.Message):
    """MsgBurn is the sdk.Msg type for allowing an admin account to burn
    a token.  For now, we only support burning from the sender account.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SENDER_FIELD_NUMBER: builtins.int
    AMOUNT_FIELD_NUMBER: builtins.int
    sender: builtins.str
    @property
    def amount(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    def __init__(
        self,
        *,
        sender: builtins.str = ...,
        amount: cosmos.base.v1beta1.coin_pb2.Coin | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["amount", b"amount"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["amount", b"amount", "sender", b"sender"]) -> None: ...

global___MsgBurn = MsgBurn

@typing_extensions.final
class MsgBurnResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___MsgBurnResponse = MsgBurnResponse

@typing_extensions.final
class MsgChangeAdmin(google.protobuf.message.Message):
    """MsgChangeAdmin is the sdk.Msg type for allowing an admin account to reassign
    adminship of a denom to a new account
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SENDER_FIELD_NUMBER: builtins.int
    DENOM_FIELD_NUMBER: builtins.int
    NEW_ADMIN_FIELD_NUMBER: builtins.int
    sender: builtins.str
    denom: builtins.str
    new_admin: builtins.str
    def __init__(
        self,
        *,
        sender: builtins.str = ...,
        denom: builtins.str = ...,
        new_admin: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["denom", b"denom", "new_admin", b"new_admin", "sender", b"sender"]) -> None: ...

global___MsgChangeAdmin = MsgChangeAdmin

@typing_extensions.final
class MsgChangeAdminResponse(google.protobuf.message.Message):
    """MsgChangeAdminResponse defines the response structure for an executed
    MsgChangeAdmin message.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___MsgChangeAdminResponse = MsgChangeAdminResponse

@typing_extensions.final
class MsgSetDenomMetadata(google.protobuf.message.Message):
    """message MsgForceTransferResponse {}

    MsgSetDenomMetadata is the sdk.Msg type for allowing an admin account to set
    the denom's bank metadata
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SENDER_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    sender: builtins.str
    @property
    def metadata(self) -> cosmos.bank.v1beta1.bank_pb2.Metadata: ...
    def __init__(
        self,
        *,
        sender: builtins.str = ...,
        metadata: cosmos.bank.v1beta1.bank_pb2.Metadata | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["metadata", b"metadata"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["metadata", b"metadata", "sender", b"sender"]) -> None: ...

global___MsgSetDenomMetadata = MsgSetDenomMetadata

@typing_extensions.final
class MsgSetDenomMetadataResponse(google.protobuf.message.Message):
    """MsgSetDenomMetadataResponse defines the response structure for an executed
    MsgSetDenomMetadata message.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___MsgSetDenomMetadataResponse = MsgSetDenomMetadataResponse



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: osmosis/pool-incentives/v1beta1/query.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google_apis.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from osmosis.incentives import gauge_pb2 as osmosis_dot_incentives_dot_gauge__pb2
from osmosis.pool_incentives.v1beta1 import incentives_pb2 as osmosis_dot_pool__incentives_dot_v1beta1_dot_incentives__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+osmosis/pool-incentives/v1beta1/query.proto\x12\x1eosmosis.poolincentives.v1beta1\x1a\x14gogoproto/gogo.proto\x1a\x1cgoogle/api/annotations.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1eosmosis/incentives/gauge.proto\x1a\x30osmosis/pool-incentives/v1beta1/incentives.proto\";\n\x14QueryGaugeIdsRequest\x12#\n\x07pool_id\x18\x01 \x01(\x04\x42\x12\xf2\xde\x1f\x0eyaml:\"pool_id\"\"\xc2\x02\n\x15QueryGaugeIdsResponse\x12\x8e\x01\n\x17gauge_ids_with_duration\x18\x01 \x03(\x0b\x32I.osmosis.poolincentives.v1beta1.QueryGaugeIdsResponse.GaugeIdWithDurationB\"\xf2\xde\x1f\x1eyaml:\"gauge_ids_with_duration\"\x1a\x97\x01\n\x13GaugeIdWithDuration\x12%\n\x08gauge_id\x18\x01 \x01(\x04\x42\x13\xf2\xde\x1f\x0fyaml:\"gauge_id\"\x12\x35\n\x08\x64uration\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationB\x08\xc8\xde\x1f\x00\x98\xdf\x1f\x01\x12\"\n\x1agauge_incentive_percentage\x18\x03 \x01(\t\"\x17\n\x15QueryDistrInfoRequest\"r\n\x16QueryDistrInfoResponse\x12X\n\ndistr_info\x18\x01 \x01(\x0b\x32).osmosis.poolincentives.v1beta1.DistrInfoB\x19\xc8\xde\x1f\x00\xf2\xde\x1f\x11yaml:\"distr_info\"\"\x14\n\x12QueryParamsRequest\"S\n\x13QueryParamsResponse\x12<\n\x06params\x18\x01 \x01(\x0b\x32&.osmosis.poolincentives.v1beta1.ParamsB\x04\xc8\xde\x1f\x00\"\x1f\n\x1dQueryLockableDurationsRequest\"~\n\x1eQueryLockableDurationsResponse\x12\\\n\x12lockable_durations\x18\x01 \x03(\x0b\x32\x19.google.protobuf.DurationB%\xc8\xde\x1f\x00\x98\xdf\x1f\x01\xf2\xde\x1f\x19yaml:\"lockable_durations\"\"\x1f\n\x1dQueryIncentivizedPoolsRequest\"\xba\x01\n\x10IncentivizedPool\x12#\n\x07pool_id\x18\x01 \x01(\x04\x42\x12\xf2\xde\x1f\x0eyaml:\"pool_id\"\x12Z\n\x11lockable_duration\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationB$\xc8\xde\x1f\x00\x98\xdf\x1f\x01\xf2\xde\x1f\x18yaml:\"lockable_duration\"\x12%\n\x08gauge_id\x18\x03 \x01(\x04\x42\x13\xf2\xde\x1f\x0fyaml:\"gauge_id\"\"\x91\x01\n\x1eQueryIncentivizedPoolsResponse\x12o\n\x12incentivized_pools\x18\x01 \x03(\x0b\x32\x30.osmosis.poolincentives.v1beta1.IncentivizedPoolB!\xc8\xde\x1f\x00\xf2\xde\x1f\x19yaml:\"incentivized_pools\"\"%\n#QueryExternalIncentiveGaugesRequest\"U\n$QueryExternalIncentiveGaugesResponse\x12-\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x19.osmosis.incentives.GaugeB\x04\xc8\xde\x1f\x00\x32\xa5\t\n\x05Query\x12\xb5\x01\n\x08GaugeIds\x12\x34.osmosis.poolincentives.v1beta1.QueryGaugeIdsRequest\x1a\x35.osmosis.poolincentives.v1beta1.QueryGaugeIdsResponse\"<\x82\xd3\xe4\x93\x02\x36\x12\x34/osmosis/pool-incentives/v1beta1/gauge-ids/{pool_id}\x12\xaf\x01\n\tDistrInfo\x12\x35.osmosis.poolincentives.v1beta1.QueryDistrInfoRequest\x1a\x36.osmosis.poolincentives.v1beta1.QueryDistrInfoResponse\"3\x82\xd3\xe4\x93\x02-\x12+/osmosis/pool-incentives/v1beta1/distr_info\x12\xa2\x01\n\x06Params\x12\x32.osmosis.poolincentives.v1beta1.QueryParamsRequest\x1a\x33.osmosis.poolincentives.v1beta1.QueryParamsResponse\"/\x82\xd3\xe4\x93\x02)\x12\'/osmosis/pool-incentives/v1beta1/params\x12\xcf\x01\n\x11LockableDurations\x12=.osmosis.poolincentives.v1beta1.QueryLockableDurationsRequest\x1a>.osmosis.poolincentives.v1beta1.QueryLockableDurationsResponse\";\x82\xd3\xe4\x93\x02\x35\x12\x33/osmosis/pool-incentives/v1beta1/lockable_durations\x12\xcf\x01\n\x11IncentivizedPools\x12=.osmosis.poolincentives.v1beta1.QueryIncentivizedPoolsRequest\x1a>.osmosis.poolincentives.v1beta1.QueryIncentivizedPoolsResponse\";\x82\xd3\xe4\x93\x02\x35\x12\x33/osmosis/pool-incentives/v1beta1/incentivized_pools\x12\xe8\x01\n\x17\x45xternalIncentiveGauges\x12\x43.osmosis.poolincentives.v1beta1.QueryExternalIncentiveGaugesRequest\x1a\x44.osmosis.poolincentives.v1beta1.QueryExternalIncentiveGaugesResponse\"B\x82\xd3\xe4\x93\x02<\x12:/osmosis/pool-incentives/v1beta1/external_incentive_gaugesB=Z;github.com/osmosis-labs/osmosis/v14/x/pool-incentives/typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'osmosis.pool_incentives.v1beta1.query_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z;github.com/osmosis-labs/osmosis/v14/x/pool-incentives/types'
  _QUERYGAUGEIDSREQUEST.fields_by_name['pool_id']._options = None
  _QUERYGAUGEIDSREQUEST.fields_by_name['pool_id']._serialized_options = b'\362\336\037\016yaml:\"pool_id\"'
  _QUERYGAUGEIDSRESPONSE_GAUGEIDWITHDURATION.fields_by_name['gauge_id']._options = None
  _QUERYGAUGEIDSRESPONSE_GAUGEIDWITHDURATION.fields_by_name['gauge_id']._serialized_options = b'\362\336\037\017yaml:\"gauge_id\"'
  _QUERYGAUGEIDSRESPONSE_GAUGEIDWITHDURATION.fields_by_name['duration']._options = None
  _QUERYGAUGEIDSRESPONSE_GAUGEIDWITHDURATION.fields_by_name['duration']._serialized_options = b'\310\336\037\000\230\337\037\001'
  _QUERYGAUGEIDSRESPONSE.fields_by_name['gauge_ids_with_duration']._options = None
  _QUERYGAUGEIDSRESPONSE.fields_by_name['gauge_ids_with_duration']._serialized_options = b'\362\336\037\036yaml:\"gauge_ids_with_duration\"'
  _QUERYDISTRINFORESPONSE.fields_by_name['distr_info']._options = None
  _QUERYDISTRINFORESPONSE.fields_by_name['distr_info']._serialized_options = b'\310\336\037\000\362\336\037\021yaml:\"distr_info\"'
  _QUERYPARAMSRESPONSE.fields_by_name['params']._options = None
  _QUERYPARAMSRESPONSE.fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _QUERYLOCKABLEDURATIONSRESPONSE.fields_by_name['lockable_durations']._options = None
  _QUERYLOCKABLEDURATIONSRESPONSE.fields_by_name['lockable_durations']._serialized_options = b'\310\336\037\000\230\337\037\001\362\336\037\031yaml:\"lockable_durations\"'
  _INCENTIVIZEDPOOL.fields_by_name['pool_id']._options = None
  _INCENTIVIZEDPOOL.fields_by_name['pool_id']._serialized_options = b'\362\336\037\016yaml:\"pool_id\"'
  _INCENTIVIZEDPOOL.fields_by_name['lockable_duration']._options = None
  _INCENTIVIZEDPOOL.fields_by_name['lockable_duration']._serialized_options = b'\310\336\037\000\230\337\037\001\362\336\037\030yaml:\"lockable_duration\"'
  _INCENTIVIZEDPOOL.fields_by_name['gauge_id']._options = None
  _INCENTIVIZEDPOOL.fields_by_name['gauge_id']._serialized_options = b'\362\336\037\017yaml:\"gauge_id\"'
  _QUERYINCENTIVIZEDPOOLSRESPONSE.fields_by_name['incentivized_pools']._options = None
  _QUERYINCENTIVIZEDPOOLSRESPONSE.fields_by_name['incentivized_pools']._serialized_options = b'\310\336\037\000\362\336\037\031yaml:\"incentivized_pools\"'
  _QUERYEXTERNALINCENTIVEGAUGESRESPONSE.fields_by_name['data']._options = None
  _QUERYEXTERNALINCENTIVEGAUGESRESPONSE.fields_by_name['data']._serialized_options = b'\310\336\037\000'
  _QUERY.methods_by_name['GaugeIds']._options = None
  _QUERY.methods_by_name['GaugeIds']._serialized_options = b'\202\323\344\223\0026\0224/osmosis/pool-incentives/v1beta1/gauge-ids/{pool_id}'
  _QUERY.methods_by_name['DistrInfo']._options = None
  _QUERY.methods_by_name['DistrInfo']._serialized_options = b'\202\323\344\223\002-\022+/osmosis/pool-incentives/v1beta1/distr_info'
  _QUERY.methods_by_name['Params']._options = None
  _QUERY.methods_by_name['Params']._serialized_options = b'\202\323\344\223\002)\022\'/osmosis/pool-incentives/v1beta1/params'
  _QUERY.methods_by_name['LockableDurations']._options = None
  _QUERY.methods_by_name['LockableDurations']._serialized_options = b'\202\323\344\223\0025\0223/osmosis/pool-incentives/v1beta1/lockable_durations'
  _QUERY.methods_by_name['IncentivizedPools']._options = None
  _QUERY.methods_by_name['IncentivizedPools']._serialized_options = b'\202\323\344\223\0025\0223/osmosis/pool-incentives/v1beta1/incentivized_pools'
  _QUERY.methods_by_name['ExternalIncentiveGauges']._options = None
  _QUERY.methods_by_name['ExternalIncentiveGauges']._serialized_options = b'\202\323\344\223\002<\022:/osmosis/pool-incentives/v1beta1/external_incentive_gauges'
  _QUERYGAUGEIDSREQUEST._serialized_start=245
  _QUERYGAUGEIDSREQUEST._serialized_end=304
  _QUERYGAUGEIDSRESPONSE._serialized_start=307
  _QUERYGAUGEIDSRESPONSE._serialized_end=629
  _QUERYGAUGEIDSRESPONSE_GAUGEIDWITHDURATION._serialized_start=478
  _QUERYGAUGEIDSRESPONSE_GAUGEIDWITHDURATION._serialized_end=629
  _QUERYDISTRINFOREQUEST._serialized_start=631
  _QUERYDISTRINFOREQUEST._serialized_end=654
  _QUERYDISTRINFORESPONSE._serialized_start=656
  _QUERYDISTRINFORESPONSE._serialized_end=770
  _QUERYPARAMSREQUEST._serialized_start=772
  _QUERYPARAMSREQUEST._serialized_end=792
  _QUERYPARAMSRESPONSE._serialized_start=794
  _QUERYPARAMSRESPONSE._serialized_end=877
  _QUERYLOCKABLEDURATIONSREQUEST._serialized_start=879
  _QUERYLOCKABLEDURATIONSREQUEST._serialized_end=910
  _QUERYLOCKABLEDURATIONSRESPONSE._serialized_start=912
  _QUERYLOCKABLEDURATIONSRESPONSE._serialized_end=1038
  _QUERYINCENTIVIZEDPOOLSREQUEST._serialized_start=1040
  _QUERYINCENTIVIZEDPOOLSREQUEST._serialized_end=1071
  _INCENTIVIZEDPOOL._serialized_start=1074
  _INCENTIVIZEDPOOL._serialized_end=1260
  _QUERYINCENTIVIZEDPOOLSRESPONSE._serialized_start=1263
  _QUERYINCENTIVIZEDPOOLSRESPONSE._serialized_end=1408
  _QUERYEXTERNALINCENTIVEGAUGESREQUEST._serialized_start=1410
  _QUERYEXTERNALINCENTIVEGAUGESREQUEST._serialized_end=1447
  _QUERYEXTERNALINCENTIVEGAUGESRESPONSE._serialized_start=1449
  _QUERYEXTERNALINCENTIVEGAUGESRESPONSE._serialized_end=1534
  _QUERY._serialized_start=1537
  _QUERY._serialized_end=2726
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import cosmos.bank.v1beta1.query_pb2
import grpc

class QueryStub:
    """Query defines the gRPC querier service."""

    def __init__(self, channel: grpc.Channel) -> None: ...
    Balance: grpc.UnaryUnaryMultiCallable[
        cosmos.bank.v1beta1.query_pb2.QueryBalanceRequest,
        cosmos.bank.v1beta1.query_pb2.QueryBalanceResponse,
    ]
    """Balance queries the balance of a single coin for a single account."""
    AllBalances: grpc.UnaryUnaryMultiCallable[
        cosmos.bank.v1beta1.query_pb2.QueryAllBalancesRequest,
        cosmos.bank.v1beta1.query_pb2.QueryAllBalancesResponse,
    ]
    """AllBalances queries the balance of all coins for a single account."""
    TotalSupply: grpc.UnaryUnaryMultiCallable[
        cosmos.bank.v1beta1.query_pb2.QueryTotalSupplyRequest,
        cosmos.bank.v1beta1.query_pb2.QueryTotalSupplyResponse,
    ]
    """TotalSupply queries the total supply of all coins."""
    SupplyOf: grpc.UnaryUnaryMultiCallable[
        cosmos.bank.v1beta1.query_pb2.QuerySupplyOfRequest,
        cosmos.bank.v1beta1.query_pb2.QuerySupplyOfResponse,
    ]
    """SupplyOf queries the supply of a single coin."""
    TotalSupplyWithoutOffset: grpc.UnaryUnaryMultiCallable[
        cosmos.bank.v1beta1.query_pb2.QueryTotalSupplyWithoutOffsetRequest,
        cosmos.bank.v1beta1.query_pb2.QueryTotalSupplyWithoutOffsetResponse,
    ]
    """TotalSupplyWithoutOffset queries the total supply of all coins."""
    SupplyOfWithoutOffset: grpc.UnaryUnaryMultiCallable[
        cosmos.bank.v1beta1.query_pb2.QuerySupplyOfWithoutOffsetRequest,
        cosmos.bank.v1beta1.query_pb2.QuerySupplyOfWithoutOffsetResponse,
    ]
    """SupplyOf queries the supply of a single coin."""
    Params: grpc.UnaryUnaryMultiCallable[
        cosmos.bank.v1beta1.query_pb2.QueryParamsRequest,
        cosmos.bank.v1beta1.query_pb2.QueryParamsResponse,
    ]
    """Params queries the parameters of x/bank module."""
    DenomMetadata: grpc.UnaryUnaryMultiCallable[
        cosmos.bank.v1beta1.query_pb2.QueryDenomMetadataRequest,
        cosmos.bank.v1beta1.query_pb2.QueryDenomMetadataResponse,
    ]
    """DenomsMetadata queries the client metadata of a given coin denomination."""
    DenomsMetadata: grpc.UnaryUnaryMultiCallable[
        cosmos.bank.v1beta1.query_pb2.QueryDenomsMetadataRequest,
        cosmos.bank.v1beta1.query_pb2.QueryDenomsMetadataResponse,
    ]
    """DenomsMetadata queries the client metadata for all registered coin denominations."""
    BaseDenom: grpc.UnaryUnaryMultiCallable[
        cosmos.bank.v1beta1.query_pb2.QueryBaseDenomRequest,
        cosmos.bank.v1beta1.query_pb2.QueryBaseDenomResponse,
    ]
    """BaseDenom queries for a base denomination given a denom that can either be
    the base denom itself or a metadata denom unit that maps to the base denom.
    """

class QueryServicer(metaclass=abc.ABCMeta):
    """Query defines the gRPC querier service."""

    @abc.abstractmethod
    def Balance(
        self,
        request: cosmos.bank.v1beta1.query_pb2.QueryBalanceRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.bank.v1beta1.query_pb2.QueryBalanceResponse:
        """Balance queries the balance of a single coin for a single account."""
    @abc.abstractmethod
    def AllBalances(
        self,
        request: cosmos.bank.v1beta1.query_pb2.QueryAllBalancesRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.bank.v1beta1.query_pb2.QueryAllBalancesResponse:
        """AllBalances queries the balance of all coins for a single account."""
    @abc.abstractmethod
    def TotalSupply(
        self,
        request: cosmos.bank.v1beta1.query_pb2.QueryTotalSupplyRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.bank.v1beta1.query_pb2.QueryTotalSupplyResponse:
        """TotalSupply queries the total supply of all coins."""
    @abc.abstractmethod
    def SupplyOf(
        self,
        request: cosmos.bank.v1beta1.query_pb2.QuerySupplyOfRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.bank.v1beta1.query_pb2.QuerySupplyOfResponse:
        """SupplyOf queries the supply of a single coin."""
    @abc.abstractmethod
    def TotalSupplyWithoutOffset(
        self,
        request: cosmos.bank.v1beta1.query_pb2.QueryTotalSupplyWithoutOffsetRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.bank.v1beta1.query_pb2.QueryTotalSupplyWithoutOffsetResponse:
        """TotalSupplyWithoutOffset queries the total supply of all coins."""
    @abc.abstractmethod
    def SupplyOfWithoutOffset(
        self,
        request: cosmos.bank.v1beta1.query_pb2.QuerySupplyOfWithoutOffsetRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.bank.v1beta1.query_pb2.QuerySupplyOfWithoutOffsetResponse:
        """SupplyOf queries the supply of a single coin."""
    @abc.abstractmethod
    def Params(
        self,
        request: cosmos.bank.v1beta1.query_pb2.QueryParamsRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.bank.v1beta1.query_pb2.QueryParamsResponse:
        """Params queries the parameters of x/bank module."""
    @abc.abstractmethod
    def DenomMetadata(
        self,
        request: cosmos.bank.v1beta1.query_pb2.QueryDenomMetadataRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.bank.v1beta1.query_pb2.QueryDenomMetadataResponse:
        """DenomsMetadata queries the client metadata of a given coin denomination."""
    @abc.abstractmethod
    def DenomsMetadata(
        self,
        request: cosmos.bank.v1beta1.query_pb2.QueryDenomsMetadataRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.bank.v1beta1.query_pb2.QueryDenomsMetadataResponse:
        """DenomsMetadata queries the client metadata for all registered coin denominations."""
    @abc.abstractmethod
    def BaseDenom(
        self,
        request: cosmos.bank.v1beta1.query_pb2.QueryBaseDenomRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.bank.v1beta1.query_pb2.QueryBaseDenomResponse:
        """BaseDenom queries for a base denomination given a denom that can either be
        the base denom itself or a metadata denom unit that maps to the base denom.
        """

def add_QueryServicer_to_server(servicer: QueryServicer, server: grpc.Server) -> None: ...



"""
Classes:
    TxClient: A client for building, simulating, and broadcasting transactions.
    Transaction: Transactions trigger state changes based on messages. Each message
        must be cryptographically signed before being broadcasted to the network.
"""
import json
import logging
from copy import deepcopy
from numbers import Number
from typing import Any, Callable, Iterable, List, Tuple, Union

from google.protobuf import any_pb2, message
from google.protobuf.json_format import MessageToDict
from osmosis_proto.proto.cosmos.base.abci.v1beta1 import abci_pb2 as abci_type
from osmosis_proto.proto.cosmos.base.v1beta1 import coin_pb2 as cosmos_base_coin_pb
from osmosis_proto.proto.cosmos.base.v1beta1.coin_pb2 import Coin
from osmosis_proto.proto.cosmos.tx.signing.v1beta1 import signing_pb2 as tx_sign
from osmosis_proto.proto.cosmos.tx.v1beta1 import tx_pb2 as cosmos_tx_type

from osmosispy import pytypes as pt
from osmosispy import wallet
from osmosispy.exceptions import SimulationError, TxError
from osmosispy.grpc_client import GrpcClient


class TxClient:
    """
    A client for building, simulating, and broadcasting transactions.
    """

    def __init__(
        self,
        priv_key: wallet.PrivateKey,
        network: pt.Network,
        client: GrpcClient,
        config: pt.TxConfig,
    ):
        self.priv_key = priv_key
        self.network = network
        self.client = client
        self.address = None
        self.config = config

    def execute_tx(
        self, tx: "Transaction", gas_estimate: float, **kwargs
    ) -> abci_type.TxResponse:
        conf: pt.TxConfig = self.get_config(**kwargs)

        def compute_gas_wanted() -> float:
            # Related to https://github.com/cosmos/cosmos-sdk/issues/14405
            # TODO We should consider adding the behavior mentioned by tac0turtle.
            gas_wanted = gas_estimate * 1.25  # apply gas multiplier
            if conf.gas_wanted > 0:
                gas_wanted = conf.gas_wanted
            elif conf.gas_multiplier > 0:
                gas_wanted = gas_estimate * conf.gas_multiplier
            return gas_wanted

        gas_wanted = compute_gas_wanted()
        gas_price = pt.GAS_PRICE if conf.gas_price <= 0 else conf.gas_price

        fee = [
            cosmos_base_coin_pb.Coin(
                amount=str(int(gas_price * gas_wanted)),
            )
        ]
        logging.info(
            "Executing transaction with fee: %s and gas_wanted: %d", fee, gas_wanted
        )
        tx = (
            tx.with_gas_limit(gas_wanted)
            .with_fee(fee)
            .with_memo("")
            .with_timeout_height(self.client.timeout_height)
        )
        tx_raw_bytes = tx.get_signed_tx_data()

        return self._send_tx(tx_raw_bytes, conf.tx_type)

    def _send_tx(self, tx_raw_bytes: bytes, tx_type: pt.TxType) -> abci_type.TxResponse:
        broadcast_fn: Callable[[bytes], abci_type.TxResponse]

        if tx_type == pt.TxType.SYNC:
            broadcast_fn = self.client.send_tx_sync_mode
        elif tx_type == pt.TxType.ASYNC:
            broadcast_fn = self.client.send_tx_async_mode
        else:
            broadcast_fn = self.client.send_tx_block_mode

        return broadcast_fn(tx_raw_bytes)

    def build_tx(
        self,
        msgs: Union[pt.PythonMsg, List[pt.PythonMsg]],
        get_sequence_from_node: bool = False,
    ) -> Tuple["Transaction", wallet.Address]:
        if not isinstance(msgs, list):
            msgs = [msgs]

        pb_msgs = [msg.to_pb() for msg in msgs]

        self.client.sync_timeout_height()
        address: wallet.Address = self.get_address_info()
        sequence: int = address.get_sequence(
            from_node=get_sequence_from_node,
            lcd_endpoint=self.network.lcd_endpoint,
        )
        tx = (
            Transaction()
            .with_messages(pb_msgs)
            .with_sequence(sequence)
            .with_account_num(address.get_number())
            .with_chain_id(self.network.chain_id)
            .with_signer(self.priv_key)
        )
        return tx, address

    def simulate(self, tx: "Transaction") -> abci_type.SimulationResponse:
        """
        Args:
            tx (Transaction): The transaction being simulated.

        Returns:
            SimulationResponse: SimulationResponse defines the response generated
                when a transaction is simulated successfully.

        Raises:
            SimulationError
        """
        sim_tx_raw_bytes: bytes = tx.get_signed_tx_data()

        sim_res: abci_type.SimulationResponse
        success: bool
        sim_res, success = self.client.simulate_tx(sim_tx_raw_bytes)
        if not success:
            raise SimulationError(sim_res)

        return sim_res

    def get_address_info(self) -> wallet.Address:
        if self.address is None:
            pub_key: wallet.PublicKey = self.priv_key.to_public_key()
            self.address = pub_key.to_address()
            self.address = self.address.init_num_seq(self.network.lcd_endpoint)

        return self.address

    def get_config(self, **kwargs) -> pt.TxConfig:
        """
        Properties in kwargs overwrite the self.config
        """
        config: pt.TxConfig = deepcopy(self.config)
        prop_names = dir(config)
        for k, v in kwargs.items():
            if k in prop_names:
                setattr(config, k, v)
            else:
                logging.warning(
                    "%s is not a supported config property, ignoring", k)

        return config


class Transaction:
    """
    Transactions trigger state changes based on messages ('msgs'). Each message
    must be signed before being broadcasted to the network, included in a block,
    validated, and approved through the consensus process.

    Attributes:
        account_num (int): Number of the account in state. Account numbers
            increment every time an account is created so that each account has
            its own number, and the highest account number is equivalent to the
            number of accounts in the 'auth' module (but not necessarily the store).
        msgs: A List of messages to be executed.
        sequence (int): A per sender "nonce" that acts as a security measure to
            prevent replay attacks on transactions. Each transaction request must
            have a different sequence number from all previously executed
            transactions so that no transaction can be replayed.
        chain_id (str): The unique identifier for the blockchain that this
            transaction targets. Inclusion of a 'chain_id' prevents potential
            attackers from using signed transactions on other blockchains.
        fee (List[Coin]): Coins to be paid in fees. The 'fee' helps prevents end
            users from spamming the network. Gas cosumed during message execution
            is typically priced from a fee equal to 'gas_consumed * gas_prices'.
            Here, 'gas_prices' is the minimum gas price, and it's a parameter local
            to each node.
        gas_limit (int): Maximum gas to be allowed for the transaction. The
            transaction execution fails if the gas limit is exceeded.
        priv_key (wallet.PrivateKey): Primary signer for the transaction. By
            convention, the signer from the first message is referred to as the
            primary signer and pays the fee for the whole transaction. We refer
            to this primary signer with 'priv_key'.
        memo (str): Memo is a note or comment to be added to the transction.
        timeout_height (int): Timeout height is the block height after which the
            transaction will not be processed by the chain.
    """

    def __init__(
        self,
        msgs: Tuple[message.Message, ...] = None,
        account_num: int = None,
        priv_key: wallet.PrivateKey = None,
        sequence: int = None,
        chain_id: str = None,
        fee: List[Coin] = None,
        gas_limit: int = 0,
        memo: str = "",
        timeout_height: int = 0,
    ):
        self.msgs = self.__convert_msgs(msgs) if msgs is not None else []
        self.account_num = account_num
        self.priv_key = priv_key
        self.sequence = sequence
        self.chain_id = chain_id
        self.fee = cosmos_tx_type.Fee(amount=fee, gas_limit=gas_limit)
        self.gas_limit = gas_limit
        self.memo = memo
        self.timeout_height = timeout_height

    @staticmethod
    def __convert_msgs(msgs: List[message.Message]) -> List[any_pb2.Any]:
        any_msgs: List[any_pb2.Any] = []
        for msg in msgs:
            any_msg = any_pb2.Any()
            any_msg.Pack(msg, type_url_prefix="")
            any_msgs.append(any_msg)
        return any_msgs

    def with_messages(self, msgs: Iterable[message.Message]) -> "Transaction":
        self.msgs.extend(self.__convert_msgs(msgs))
        return self

    def with_sender(self, client: GrpcClient, sender: str) -> "Transaction":
        if len(self.msgs) == 0:
            raise IndexError(
                "messsage is empty, please use with_messages at least 1 message"
            )
        account = client.get_account(sender)
        if account:
            self.account_num = account.account_number
            self.sequence = account.sequence
            return self
        raise KeyError("Account doesn't exist")

    def with_signer(self, priv_key: wallet.PrivateKey):
        self.priv_key = priv_key
        return self

    def with_account_num(self, account_num: int) -> "Transaction":
        self.account_num = account_num
        return self

    def with_sequence(self, sequence: int) -> "Transaction":
        self.sequence = sequence
        return self

    def with_chain_id(self, chain_id: str) -> "Transaction":
        self.chain_id = chain_id
        return self

    def with_fee(self, fee: List[Coin]) -> "Transaction":
        self.fee = cosmos_tx_type.Fee(amount=fee, gas_limit=self.fee.gas_limit)
        return self

    def with_gas_limit(self, gas: Number) -> "Transaction":
        self.fee.gas_limit = int(gas)
        return self

    def with_memo(self, memo: str) -> "Transaction":
        if len(memo) > pt.MAX_MEMO_CHARACTERS:
            raise ValueError("memo is too large")
        self.memo = memo
        return self

    def with_timeout_height(self, timeout_height: int) -> "Transaction":
        self.timeout_height = timeout_height
        return self

    def __generate_info(self, public_key: wallet.PublicKey = None) -> Tuple[str, str]:
        body = cosmos_tx_type.TxBody(
            messages=self.msgs, memo=self.memo, timeout_height=self.timeout_height
        )

        body_bytes = body.SerializeToString()
        mode_info = cosmos_tx_type.ModeInfo(
            single=cosmos_tx_type.ModeInfo.Single(
                mode=tx_sign.SIGN_MODE_DIRECT)
        )

        if public_key:
            any_public_key = any_pb2.Any()
            any_public_key.Pack(
                public_key.to_public_key_proto(), type_url_prefix="")
            signer_info = cosmos_tx_type.SignerInfo(
                mode_info=mode_info, sequence=self.sequence, public_key=any_public_key
            )
        else:
            signer_info = cosmos_tx_type.SignerInfo(
                mode_info=mode_info, sequence=self.sequence
            )

        auth_info = cosmos_tx_type.AuthInfo(
            signer_infos=[signer_info], fee=self.fee)
        auth_info_bytes = auth_info.SerializeToString()

        return body_bytes, auth_info_bytes

    def get_sign_doc(
        self, public_key: wallet.PublicKey = None
    ) -> cosmos_tx_type.SignDoc:
        if len(self.msgs) == 0:
            raise ValueError("message is empty")

        if self.account_num is None:
            raise RuntimeError("account_num should be defined")

        if self.sequence is None:
            raise RuntimeError("sequence should be defined")

        if self.chain_id is None:
            raise RuntimeError("chain_id should be defined")

        body_bytes, auth_info_bytes = self.__generate_info(public_key)

        return cosmos_tx_type.SignDoc(
            body_bytes=body_bytes,
            auth_info_bytes=auth_info_bytes,
            chain_id=self.chain_id,
            account_number=self.account_num,
        )

    def get_tx_data(
        self, signature: bytes, public_key: wallet.PublicKey = None
    ) -> bytes:
        body_bytes, auth_info_bytes = self.__generate_info(public_key)

        tx_raw = cosmos_tx_type.TxRaw(
            body_bytes=body_bytes,
            auth_info_bytes=auth_info_bytes,
            signatures=[signature],
        )
        return tx_raw.SerializeToString()

    def get_signed_tx_data(self) -> bytes:
        if self.priv_key is None:
            raise RuntimeError("priv_key should be defined")

        pub_key = self.priv_key.to_public_key()
        sign_doc = self.get_sign_doc(pub_key)
        sig = self.priv_key.sign(sign_doc.SerializeToString())
        return self.get_tx_data(sig, pub_key)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.duration_pb2
import google.protobuf.internal.containers
import google.protobuf.message
import osmosis.lockup.lock_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class MsgLockTokens(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OWNER_FIELD_NUMBER: builtins.int
    DURATION_FIELD_NUMBER: builtins.int
    COINS_FIELD_NUMBER: builtins.int
    owner: builtins.str
    @property
    def duration(self) -> google.protobuf.duration_pb2.Duration: ...
    @property
    def coins(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(
        self,
        *,
        owner: builtins.str = ...,
        duration: google.protobuf.duration_pb2.Duration | None = ...,
        coins: collections.abc.Iterable[cosmos.base.v1beta1.coin_pb2.Coin] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["duration", b"duration"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["coins", b"coins", "duration", b"duration", "owner", b"owner"]) -> None: ...

global___MsgLockTokens = MsgLockTokens

@typing_extensions.final
class MsgLockTokensResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    ID: builtins.int
    def __init__(
        self,
        *,
        ID: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["ID", b"ID"]) -> None: ...

global___MsgLockTokensResponse = MsgLockTokensResponse

@typing_extensions.final
class MsgBeginUnlockingAll(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OWNER_FIELD_NUMBER: builtins.int
    owner: builtins.str
    def __init__(
        self,
        *,
        owner: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["owner", b"owner"]) -> None: ...

global___MsgBeginUnlockingAll = MsgBeginUnlockingAll

@typing_extensions.final
class MsgBeginUnlockingAllResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UNLOCKS_FIELD_NUMBER: builtins.int
    @property
    def unlocks(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[osmosis.lockup.lock_pb2.PeriodLock]: ...
    def __init__(
        self,
        *,
        unlocks: collections.abc.Iterable[osmosis.lockup.lock_pb2.PeriodLock] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["unlocks", b"unlocks"]) -> None: ...

global___MsgBeginUnlockingAllResponse = MsgBeginUnlockingAllResponse

@typing_extensions.final
class MsgBeginUnlocking(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OWNER_FIELD_NUMBER: builtins.int
    ID_FIELD_NUMBER: builtins.int
    COINS_FIELD_NUMBER: builtins.int
    owner: builtins.str
    ID: builtins.int
    @property
    def coins(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]:
        """Amount of unlocking coins. Unlock all if not set."""
    def __init__(
        self,
        *,
        owner: builtins.str = ...,
        ID: builtins.int = ...,
        coins: collections.abc.Iterable[cosmos.base.v1beta1.coin_pb2.Coin] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["ID", b"ID", "coins", b"coins", "owner", b"owner"]) -> None: ...

global___MsgBeginUnlocking = MsgBeginUnlocking

@typing_extensions.final
class MsgBeginUnlockingResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SUCCESS_FIELD_NUMBER: builtins.int
    success: builtins.bool
    def __init__(
        self,
        *,
        success: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["success", b"success"]) -> None: ...

global___MsgBeginUnlockingResponse = MsgBeginUnlockingResponse

@typing_extensions.final
class MsgExtendLockup(google.protobuf.message.Message):
    """MsgExtendLockup extends the existing lockup's duration.
    The new duration is longer than the original.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OWNER_FIELD_NUMBER: builtins.int
    ID_FIELD_NUMBER: builtins.int
    DURATION_FIELD_NUMBER: builtins.int
    owner: builtins.str
    ID: builtins.int
    @property
    def duration(self) -> google.protobuf.duration_pb2.Duration:
        """duration to be set. fails if lower than the current duration, or is
        unlocking
        """
    def __init__(
        self,
        *,
        owner: builtins.str = ...,
        ID: builtins.int = ...,
        duration: google.protobuf.duration_pb2.Duration | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["duration", b"duration"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["ID", b"ID", "duration", b"duration", "owner", b"owner"]) -> None: ...

global___MsgExtendLockup = MsgExtendLockup

@typing_extensions.final
class MsgExtendLockupResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SUCCESS_FIELD_NUMBER: builtins.int
    success: builtins.bool
    def __init__(
        self,
        *,
        success: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["success", b"success"]) -> None: ...

global___MsgExtendLockupResponse = MsgExtendLockupResponse

@typing_extensions.final
class MsgForceUnlock(google.protobuf.message.Message):
    """MsgForceUnlock unlocks locks immediately for
    addresses registered via governance.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OWNER_FIELD_NUMBER: builtins.int
    ID_FIELD_NUMBER: builtins.int
    COINS_FIELD_NUMBER: builtins.int
    owner: builtins.str
    ID: builtins.int
    @property
    def coins(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]:
        """Amount of unlocking coins. Unlock all if not set."""
    def __init__(
        self,
        *,
        owner: builtins.str = ...,
        ID: builtins.int = ...,
        coins: collections.abc.Iterable[cosmos.base.v1beta1.coin_pb2.Coin] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["ID", b"ID", "coins", b"coins", "owner", b"owner"]) -> None: ...

global___MsgForceUnlock = MsgForceUnlock

@typing_extensions.final
class MsgForceUnlockResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SUCCESS_FIELD_NUMBER: builtins.int
    success: builtins.bool
    def __init__(
        self,
        *,
        success: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["success", b"success"]) -> None: ...

global___MsgForceUnlockResponse = MsgForceUnlockResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class Snapshot(google.protobuf.message.Message):
    """Snapshot contains Tendermint state sync snapshot info."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HEIGHT_FIELD_NUMBER: builtins.int
    FORMAT_FIELD_NUMBER: builtins.int
    CHUNKS_FIELD_NUMBER: builtins.int
    HASH_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    height: builtins.int
    format: builtins.int
    chunks: builtins.int
    hash: builtins.bytes
    @property
    def metadata(self) -> global___Metadata: ...
    def __init__(
        self,
        *,
        height: builtins.int = ...,
        format: builtins.int = ...,
        chunks: builtins.int = ...,
        hash: builtins.bytes = ...,
        metadata: global___Metadata | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["metadata", b"metadata"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["chunks", b"chunks", "format", b"format", "hash", b"hash", "height", b"height", "metadata", b"metadata"]) -> None: ...

global___Snapshot = Snapshot

@typing_extensions.final
class Metadata(google.protobuf.message.Message):
    """Metadata contains SDK-specific snapshot metadata."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CHUNK_HASHES_FIELD_NUMBER: builtins.int
    @property
    def chunk_hashes(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bytes]:
        """SHA-256 chunk hashes"""
    def __init__(
        self,
        *,
        chunk_hashes: collections.abc.Iterable[builtins.bytes] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["chunk_hashes", b"chunk_hashes"]) -> None: ...

global___Metadata = Metadata

@typing_extensions.final
class SnapshotItem(google.protobuf.message.Message):
    """SnapshotItem is an item contained in a rootmulti.Store snapshot."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STORE_FIELD_NUMBER: builtins.int
    IAVL_FIELD_NUMBER: builtins.int
    EXTENSION_FIELD_NUMBER: builtins.int
    EXTENSION_PAYLOAD_FIELD_NUMBER: builtins.int
    APP_VERSION_FIELD_NUMBER: builtins.int
    @property
    def store(self) -> global___SnapshotStoreItem: ...
    @property
    def iavl(self) -> global___SnapshotIAVLItem: ...
    @property
    def extension(self) -> global___SnapshotExtensionMeta: ...
    @property
    def extension_payload(self) -> global___SnapshotExtensionPayload: ...
    @property
    def app_version(self) -> global___SnapshotAppVersion: ...
    def __init__(
        self,
        *,
        store: global___SnapshotStoreItem | None = ...,
        iavl: global___SnapshotIAVLItem | None = ...,
        extension: global___SnapshotExtensionMeta | None = ...,
        extension_payload: global___SnapshotExtensionPayload | None = ...,
        app_version: global___SnapshotAppVersion | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["app_version", b"app_version", "extension", b"extension", "extension_payload", b"extension_payload", "iavl", b"iavl", "item", b"item", "store", b"store"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["app_version", b"app_version", "extension", b"extension", "extension_payload", b"extension_payload", "iavl", b"iavl", "item", b"item", "store", b"store"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["item", b"item"]) -> typing_extensions.Literal["store", "iavl", "extension", "extension_payload", "app_version"] | None: ...

global___SnapshotItem = SnapshotItem

@typing_extensions.final
class SnapshotStoreItem(google.protobuf.message.Message):
    """SnapshotStoreItem contains metadata about a snapshotted store."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    name: builtins.str
    def __init__(
        self,
        *,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name"]) -> None: ...

global___SnapshotStoreItem = SnapshotStoreItem

@typing_extensions.final
class SnapshotIAVLItem(google.protobuf.message.Message):
    """SnapshotIAVLItem is an exported IAVL node."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KEY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    key: builtins.bytes
    value: builtins.bytes
    version: builtins.int
    """version is block height"""
    height: builtins.int
    """height is depth of the tree."""
    def __init__(
        self,
        *,
        key: builtins.bytes = ...,
        value: builtins.bytes = ...,
        version: builtins.int = ...,
        height: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["height", b"height", "key", b"key", "value", b"value", "version", b"version"]) -> None: ...

global___SnapshotIAVLItem = SnapshotIAVLItem

@typing_extensions.final
class SnapshotExtensionMeta(google.protobuf.message.Message):
    """SnapshotExtensionMeta contains metadata about an external snapshotter."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    FORMAT_FIELD_NUMBER: builtins.int
    name: builtins.str
    format: builtins.int
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        format: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["format", b"format", "name", b"name"]) -> None: ...

global___SnapshotExtensionMeta = SnapshotExtensionMeta

@typing_extensions.final
class SnapshotExtensionPayload(google.protobuf.message.Message):
    """SnapshotExtensionPayload contains payloads of an external snapshotter."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PAYLOAD_FIELD_NUMBER: builtins.int
    payload: builtins.bytes
    def __init__(
        self,
        *,
        payload: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["payload", b"payload"]) -> None: ...

global___SnapshotExtensionPayload = SnapshotExtensionPayload

@typing_extensions.final
class SnapshotAppVersion(google.protobuf.message.Message):
    """SnapshotAppVersion contains the app version."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VERSION_FIELD_NUMBER: builtins.int
    version: builtins.int
    def __init__(
        self,
        *,
        version: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["version", b"version"]) -> None: ...

global___SnapshotAppVersion = SnapshotAppVersion



import collections
import pprint
from typing import Dict, List


class RawEvent(collections.abc.MutableMapping):
    """Dictionary representing a Tendermint event. In the raw TxOutput of a
    successful transaction, it's the value at

    ```python
    tx_output['rawLog'][0]['events']
    ```

    ### Keys (KeyType):
    - attributes (List[Dict[str,str]])
    - type (str)

    ### Example:
    ```python
    {'attributes': [
        {'key': 'recipient', 'value': 'osmo1uvu52rxwqj5ndmm59y6atvx33mru9xrz6sqekr'},
        {'key': 'sender', 'value': 'osmo1zaavvzxez0elundtn32qnk9lkm8kmcsz44g7xl'},
        {'key': 'amount', 'value': '7uosmo,70unusd'}],
    'type': 'transfer'}
    ```
    """


class Event:
    """A Tendermint event. An event contains a type and set of attributes.
    Events allow application developers to attach additional information to the
    'ResponseBeginBlock', 'ResponseEndBlock', 'ResponseCheckTx', and 'ResponseDeliverTx'
    functions in the ABCI (application blockchain interface).

    In the Tendermint protobuf, the hard definition is:

    ```proto
    message Event {
      string type = 1;
      repeated EventAttribute attributes = 2;
    }
    message EventAttribute {
      bytes key = 1;
      bytes value = 2;
      bool index = 3;
    }
    ```

    - Ref: [cosmos-sdk/types/events.go](https://github.com/cosmos/cosmos-sdk/blob/93abfdd21d9892550da315b10308519b43fb1775/types/events.go#L221)
    - Ref: [tendermint/tendermint/proto/tendermint/abci/types.proto](https://github.com/tendermint/tendermint/blob/a6dd0d270abc3c01f223eedee44d8b285ae273f6/proto/tendermint/abci/types.proto)
    """

    type: str
    attrs: Dict[str, str]

    def __init__(self, raw_event: RawEvent):
        self.type = raw_event["type"]
        self.attrs = self.parse_attributes(raw_event["attributes"])

    @staticmethod
    def parse_attributes(raw_attributes: List[Dict[str, str]]) -> Dict[str, str]:
        try:
            attributes: dict[str, str] = {
                kv_dict['key']: kv_dict['value'] for kv_dict in raw_attributes
            }
            return attributes
        except:
            raise Exception(
                f"failed to parse raw attributes:\n{pprint.pformat(raw_attributes)}"
            )

    def __repr__(self) -> str:
        return f"Event(type={self.type}, attrs={self.attrs})"

    def to_dict(self) -> Dict[str, Dict[str, str]]:
        """
        Returns as dictionary

        Returns:
            Dict[str, Dict[str, str]]: the dictionary

        """
        return {self.type: self.attrs}


class TxLogEvents:
    """An element of 'TxResp.rawLog'. This object contains events and messages.

    Keys (KeyType):
        type (str)
        attributes (List[EventAttribute])

    Args:
        events_raw (List[RawEvent])

    Attributes:
        events (List[Event])
        msgs (List[str])
        events_raw (List[RawEvent])
        event_types (List[str])
    """

    events: List[Event]
    msgs: List[str]
    events_raw: List[RawEvent]
    event_types: List[str]

    def __init__(self, events_raw: List[RawEvent] = []):
        self.events_raw = events_raw
        self.events = [Event(raw_event) for raw_event in events_raw]
        self.msgs = self.get_msg_types()

    def get_msg_types(self) -> List[str]:
        """
        Returns the message types as a list of strings.

        Returns:
            List[str]: the list of msg types as strings

        """
        msgs = []
        self.event_types = []
        for event in self.events:
            self.event_types.append(event.type)
            if event.type == "message":
                msgs.append(event.attrs["action"])
        return msgs

    def __repr__(self) -> str:
        self_as_dict = dict(msgs=self.msgs, events=[
                            e.to_dict() for e in self.events])
        return pprint.pformat(self_as_dict, indent=2)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class MsgSubmitEvidence(google.protobuf.message.Message):
    """MsgSubmitEvidence represents a message that supports submitting arbitrary
    Evidence of misbehavior such as equivocation or counterfactual signing.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SUBMITTER_FIELD_NUMBER: builtins.int
    EVIDENCE_FIELD_NUMBER: builtins.int
    submitter: builtins.str
    @property
    def evidence(self) -> google.protobuf.any_pb2.Any: ...
    def __init__(
        self,
        *,
        submitter: builtins.str = ...,
        evidence: google.protobuf.any_pb2.Any | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["evidence", b"evidence"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["evidence", b"evidence", "submitter", b"submitter"]) -> None: ...

global___MsgSubmitEvidence = MsgSubmitEvidence

@typing_extensions.final
class MsgSubmitEvidenceResponse(google.protobuf.message.Message):
    """MsgSubmitEvidenceResponse defines the Msg/SubmitEvidence response type."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HASH_FIELD_NUMBER: builtins.int
    hash: builtins.bytes
    """hash defines the hash of the evidence."""
    def __init__(
        self,
        *,
        hash: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["hash", b"hash"]) -> None: ...

global___MsgSubmitEvidenceResponse = MsgSubmitEvidenceResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys
import tendermint.crypto.keys_pb2
import tendermint.crypto.proof_pb2
import tendermint.types.params_pb2
import tendermint.types.types_pb2
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _CheckTxType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _CheckTxTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_CheckTxType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    NEW: _CheckTxType.ValueType  # 0
    RECHECK: _CheckTxType.ValueType  # 1

class CheckTxType(_CheckTxType, metaclass=_CheckTxTypeEnumTypeWrapper): ...

NEW: CheckTxType.ValueType  # 0
RECHECK: CheckTxType.ValueType  # 1
global___CheckTxType = CheckTxType

class _EvidenceType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _EvidenceTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_EvidenceType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNKNOWN: _EvidenceType.ValueType  # 0
    DUPLICATE_VOTE: _EvidenceType.ValueType  # 1
    LIGHT_CLIENT_ATTACK: _EvidenceType.ValueType  # 2

class EvidenceType(_EvidenceType, metaclass=_EvidenceTypeEnumTypeWrapper): ...

UNKNOWN: EvidenceType.ValueType  # 0
DUPLICATE_VOTE: EvidenceType.ValueType  # 1
LIGHT_CLIENT_ATTACK: EvidenceType.ValueType  # 2
global___EvidenceType = EvidenceType

@typing_extensions.final
class Request(google.protobuf.message.Message):
    """----------------------------------------
    Request types
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ECHO_FIELD_NUMBER: builtins.int
    FLUSH_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    SET_OPTION_FIELD_NUMBER: builtins.int
    INIT_CHAIN_FIELD_NUMBER: builtins.int
    QUERY_FIELD_NUMBER: builtins.int
    BEGIN_BLOCK_FIELD_NUMBER: builtins.int
    CHECK_TX_FIELD_NUMBER: builtins.int
    DELIVER_TX_FIELD_NUMBER: builtins.int
    END_BLOCK_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    LIST_SNAPSHOTS_FIELD_NUMBER: builtins.int
    OFFER_SNAPSHOT_FIELD_NUMBER: builtins.int
    LOAD_SNAPSHOT_CHUNK_FIELD_NUMBER: builtins.int
    APPLY_SNAPSHOT_CHUNK_FIELD_NUMBER: builtins.int
    @property
    def echo(self) -> global___RequestEcho: ...
    @property
    def flush(self) -> global___RequestFlush: ...
    @property
    def info(self) -> global___RequestInfo: ...
    @property
    def set_option(self) -> global___RequestSetOption: ...
    @property
    def init_chain(self) -> global___RequestInitChain: ...
    @property
    def query(self) -> global___RequestQuery: ...
    @property
    def begin_block(self) -> global___RequestBeginBlock: ...
    @property
    def check_tx(self) -> global___RequestCheckTx: ...
    @property
    def deliver_tx(self) -> global___RequestDeliverTx: ...
    @property
    def end_block(self) -> global___RequestEndBlock: ...
    @property
    def commit(self) -> global___RequestCommit: ...
    @property
    def list_snapshots(self) -> global___RequestListSnapshots: ...
    @property
    def offer_snapshot(self) -> global___RequestOfferSnapshot: ...
    @property
    def load_snapshot_chunk(self) -> global___RequestLoadSnapshotChunk: ...
    @property
    def apply_snapshot_chunk(self) -> global___RequestApplySnapshotChunk: ...
    def __init__(
        self,
        *,
        echo: global___RequestEcho | None = ...,
        flush: global___RequestFlush | None = ...,
        info: global___RequestInfo | None = ...,
        set_option: global___RequestSetOption | None = ...,
        init_chain: global___RequestInitChain | None = ...,
        query: global___RequestQuery | None = ...,
        begin_block: global___RequestBeginBlock | None = ...,
        check_tx: global___RequestCheckTx | None = ...,
        deliver_tx: global___RequestDeliverTx | None = ...,
        end_block: global___RequestEndBlock | None = ...,
        commit: global___RequestCommit | None = ...,
        list_snapshots: global___RequestListSnapshots | None = ...,
        offer_snapshot: global___RequestOfferSnapshot | None = ...,
        load_snapshot_chunk: global___RequestLoadSnapshotChunk | None = ...,
        apply_snapshot_chunk: global___RequestApplySnapshotChunk | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["apply_snapshot_chunk", b"apply_snapshot_chunk", "begin_block", b"begin_block", "check_tx", b"check_tx", "commit", b"commit", "deliver_tx", b"deliver_tx", "echo", b"echo", "end_block", b"end_block", "flush", b"flush", "info", b"info", "init_chain", b"init_chain", "list_snapshots", b"list_snapshots", "load_snapshot_chunk", b"load_snapshot_chunk", "offer_snapshot", b"offer_snapshot", "query", b"query", "set_option", b"set_option", "value", b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["apply_snapshot_chunk", b"apply_snapshot_chunk", "begin_block", b"begin_block", "check_tx", b"check_tx", "commit", b"commit", "deliver_tx", b"deliver_tx", "echo", b"echo", "end_block", b"end_block", "flush", b"flush", "info", b"info", "init_chain", b"init_chain", "list_snapshots", b"list_snapshots", "load_snapshot_chunk", b"load_snapshot_chunk", "offer_snapshot", b"offer_snapshot", "query", b"query", "set_option", b"set_option", "value", b"value"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["value", b"value"]) -> typing_extensions.Literal["echo", "flush", "info", "set_option", "init_chain", "query", "begin_block", "check_tx", "deliver_tx", "end_block", "commit", "list_snapshots", "offer_snapshot", "load_snapshot_chunk", "apply_snapshot_chunk"] | None: ...

global___Request = Request

@typing_extensions.final
class RequestEcho(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MESSAGE_FIELD_NUMBER: builtins.int
    message: builtins.str
    def __init__(
        self,
        *,
        message: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["message", b"message"]) -> None: ...

global___RequestEcho = RequestEcho

@typing_extensions.final
class RequestFlush(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___RequestFlush = RequestFlush

@typing_extensions.final
class RequestInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VERSION_FIELD_NUMBER: builtins.int
    BLOCK_VERSION_FIELD_NUMBER: builtins.int
    P2P_VERSION_FIELD_NUMBER: builtins.int
    version: builtins.str
    block_version: builtins.int
    p2p_version: builtins.int
    def __init__(
        self,
        *,
        version: builtins.str = ...,
        block_version: builtins.int = ...,
        p2p_version: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["block_version", b"block_version", "p2p_version", b"p2p_version", "version", b"version"]) -> None: ...

global___RequestInfo = RequestInfo

@typing_extensions.final
class RequestSetOption(google.protobuf.message.Message):
    """nondeterministic"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KEY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    key: builtins.str
    value: builtins.str
    def __init__(
        self,
        *,
        key: builtins.str = ...,
        value: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

global___RequestSetOption = RequestSetOption

@typing_extensions.final
class RequestInitChain(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TIME_FIELD_NUMBER: builtins.int
    CHAIN_ID_FIELD_NUMBER: builtins.int
    CONSENSUS_PARAMS_FIELD_NUMBER: builtins.int
    VALIDATORS_FIELD_NUMBER: builtins.int
    APP_STATE_BYTES_FIELD_NUMBER: builtins.int
    INITIAL_HEIGHT_FIELD_NUMBER: builtins.int
    @property
    def time(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    chain_id: builtins.str
    @property
    def consensus_params(self) -> global___ConsensusParams: ...
    @property
    def validators(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ValidatorUpdate]: ...
    app_state_bytes: builtins.bytes
    initial_height: builtins.int
    def __init__(
        self,
        *,
        time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        chain_id: builtins.str = ...,
        consensus_params: global___ConsensusParams | None = ...,
        validators: collections.abc.Iterable[global___ValidatorUpdate] | None = ...,
        app_state_bytes: builtins.bytes = ...,
        initial_height: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["consensus_params", b"consensus_params", "time", b"time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["app_state_bytes", b"app_state_bytes", "chain_id", b"chain_id", "consensus_params", b"consensus_params", "initial_height", b"initial_height", "time", b"time", "validators", b"validators"]) -> None: ...

global___RequestInitChain = RequestInitChain

@typing_extensions.final
class RequestQuery(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_FIELD_NUMBER: builtins.int
    PATH_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    PROVE_FIELD_NUMBER: builtins.int
    data: builtins.bytes
    path: builtins.str
    height: builtins.int
    prove: builtins.bool
    def __init__(
        self,
        *,
        data: builtins.bytes = ...,
        path: builtins.str = ...,
        height: builtins.int = ...,
        prove: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["data", b"data", "height", b"height", "path", b"path", "prove", b"prove"]) -> None: ...

global___RequestQuery = RequestQuery

@typing_extensions.final
class RequestBeginBlock(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HASH_FIELD_NUMBER: builtins.int
    HEADER_FIELD_NUMBER: builtins.int
    LAST_COMMIT_INFO_FIELD_NUMBER: builtins.int
    BYZANTINE_VALIDATORS_FIELD_NUMBER: builtins.int
    hash: builtins.bytes
    @property
    def header(self) -> tendermint.types.types_pb2.Header: ...
    @property
    def last_commit_info(self) -> global___LastCommitInfo: ...
    @property
    def byzantine_validators(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Evidence]: ...
    def __init__(
        self,
        *,
        hash: builtins.bytes = ...,
        header: tendermint.types.types_pb2.Header | None = ...,
        last_commit_info: global___LastCommitInfo | None = ...,
        byzantine_validators: collections.abc.Iterable[global___Evidence] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["header", b"header", "last_commit_info", b"last_commit_info"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["byzantine_validators", b"byzantine_validators", "hash", b"hash", "header", b"header", "last_commit_info", b"last_commit_info"]) -> None: ...

global___RequestBeginBlock = RequestBeginBlock

@typing_extensions.final
class RequestCheckTx(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TX_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    tx: builtins.bytes
    type: global___CheckTxType.ValueType
    def __init__(
        self,
        *,
        tx: builtins.bytes = ...,
        type: global___CheckTxType.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["tx", b"tx", "type", b"type"]) -> None: ...

global___RequestCheckTx = RequestCheckTx

@typing_extensions.final
class RequestDeliverTx(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TX_FIELD_NUMBER: builtins.int
    tx: builtins.bytes
    def __init__(
        self,
        *,
        tx: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["tx", b"tx"]) -> None: ...

global___RequestDeliverTx = RequestDeliverTx

@typing_extensions.final
class RequestEndBlock(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HEIGHT_FIELD_NUMBER: builtins.int
    height: builtins.int
    def __init__(
        self,
        *,
        height: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["height", b"height"]) -> None: ...

global___RequestEndBlock = RequestEndBlock

@typing_extensions.final
class RequestCommit(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___RequestCommit = RequestCommit

@typing_extensions.final
class RequestListSnapshots(google.protobuf.message.Message):
    """lists available snapshots"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___RequestListSnapshots = RequestListSnapshots

@typing_extensions.final
class RequestOfferSnapshot(google.protobuf.message.Message):
    """offers a snapshot to the application"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SNAPSHOT_FIELD_NUMBER: builtins.int
    APP_HASH_FIELD_NUMBER: builtins.int
    @property
    def snapshot(self) -> global___Snapshot:
        """snapshot offered by peers"""
    app_hash: builtins.bytes
    """light client-verified app hash for snapshot height"""
    def __init__(
        self,
        *,
        snapshot: global___Snapshot | None = ...,
        app_hash: builtins.bytes = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["snapshot", b"snapshot"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["app_hash", b"app_hash", "snapshot", b"snapshot"]) -> None: ...

global___RequestOfferSnapshot = RequestOfferSnapshot

@typing_extensions.final
class RequestLoadSnapshotChunk(google.protobuf.message.Message):
    """loads a snapshot chunk"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HEIGHT_FIELD_NUMBER: builtins.int
    FORMAT_FIELD_NUMBER: builtins.int
    CHUNK_FIELD_NUMBER: builtins.int
    height: builtins.int
    format: builtins.int
    chunk: builtins.int
    def __init__(
        self,
        *,
        height: builtins.int = ...,
        format: builtins.int = ...,
        chunk: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["chunk", b"chunk", "format", b"format", "height", b"height"]) -> None: ...

global___RequestLoadSnapshotChunk = RequestLoadSnapshotChunk

@typing_extensions.final
class RequestApplySnapshotChunk(google.protobuf.message.Message):
    """Applies a snapshot chunk"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INDEX_FIELD_NUMBER: builtins.int
    CHUNK_FIELD_NUMBER: builtins.int
    SENDER_FIELD_NUMBER: builtins.int
    index: builtins.int
    chunk: builtins.bytes
    sender: builtins.str
    def __init__(
        self,
        *,
        index: builtins.int = ...,
        chunk: builtins.bytes = ...,
        sender: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["chunk", b"chunk", "index", b"index", "sender", b"sender"]) -> None: ...

global___RequestApplySnapshotChunk = RequestApplySnapshotChunk

@typing_extensions.final
class Response(google.protobuf.message.Message):
    """----------------------------------------
    Response types
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EXCEPTION_FIELD_NUMBER: builtins.int
    ECHO_FIELD_NUMBER: builtins.int
    FLUSH_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    SET_OPTION_FIELD_NUMBER: builtins.int
    INIT_CHAIN_FIELD_NUMBER: builtins.int
    QUERY_FIELD_NUMBER: builtins.int
    BEGIN_BLOCK_FIELD_NUMBER: builtins.int
    CHECK_TX_FIELD_NUMBER: builtins.int
    DELIVER_TX_FIELD_NUMBER: builtins.int
    END_BLOCK_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    LIST_SNAPSHOTS_FIELD_NUMBER: builtins.int
    OFFER_SNAPSHOT_FIELD_NUMBER: builtins.int
    LOAD_SNAPSHOT_CHUNK_FIELD_NUMBER: builtins.int
    APPLY_SNAPSHOT_CHUNK_FIELD_NUMBER: builtins.int
    @property
    def exception(self) -> global___ResponseException: ...
    @property
    def echo(self) -> global___ResponseEcho: ...
    @property
    def flush(self) -> global___ResponseFlush: ...
    @property
    def info(self) -> global___ResponseInfo: ...
    @property
    def set_option(self) -> global___ResponseSetOption: ...
    @property
    def init_chain(self) -> global___ResponseInitChain: ...
    @property
    def query(self) -> global___ResponseQuery: ...
    @property
    def begin_block(self) -> global___ResponseBeginBlock: ...
    @property
    def check_tx(self) -> global___ResponseCheckTx: ...
    @property
    def deliver_tx(self) -> global___ResponseDeliverTx: ...
    @property
    def end_block(self) -> global___ResponseEndBlock: ...
    @property
    def commit(self) -> global___ResponseCommit: ...
    @property
    def list_snapshots(self) -> global___ResponseListSnapshots: ...
    @property
    def offer_snapshot(self) -> global___ResponseOfferSnapshot: ...
    @property
    def load_snapshot_chunk(self) -> global___ResponseLoadSnapshotChunk: ...
    @property
    def apply_snapshot_chunk(self) -> global___ResponseApplySnapshotChunk: ...
    def __init__(
        self,
        *,
        exception: global___ResponseException | None = ...,
        echo: global___ResponseEcho | None = ...,
        flush: global___ResponseFlush | None = ...,
        info: global___ResponseInfo | None = ...,
        set_option: global___ResponseSetOption | None = ...,
        init_chain: global___ResponseInitChain | None = ...,
        query: global___ResponseQuery | None = ...,
        begin_block: global___ResponseBeginBlock | None = ...,
        check_tx: global___ResponseCheckTx | None = ...,
        deliver_tx: global___ResponseDeliverTx | None = ...,
        end_block: global___ResponseEndBlock | None = ...,
        commit: global___ResponseCommit | None = ...,
        list_snapshots: global___ResponseListSnapshots | None = ...,
        offer_snapshot: global___ResponseOfferSnapshot | None = ...,
        load_snapshot_chunk: global___ResponseLoadSnapshotChunk | None = ...,
        apply_snapshot_chunk: global___ResponseApplySnapshotChunk | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["apply_snapshot_chunk", b"apply_snapshot_chunk", "begin_block", b"begin_block", "check_tx", b"check_tx", "commit", b"commit", "deliver_tx", b"deliver_tx", "echo", b"echo", "end_block", b"end_block", "exception", b"exception", "flush", b"flush", "info", b"info", "init_chain", b"init_chain", "list_snapshots", b"list_snapshots", "load_snapshot_chunk", b"load_snapshot_chunk", "offer_snapshot", b"offer_snapshot", "query", b"query", "set_option", b"set_option", "value", b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["apply_snapshot_chunk", b"apply_snapshot_chunk", "begin_block", b"begin_block", "check_tx", b"check_tx", "commit", b"commit", "deliver_tx", b"deliver_tx", "echo", b"echo", "end_block", b"end_block", "exception", b"exception", "flush", b"flush", "info", b"info", "init_chain", b"init_chain", "list_snapshots", b"list_snapshots", "load_snapshot_chunk", b"load_snapshot_chunk", "offer_snapshot", b"offer_snapshot", "query", b"query", "set_option", b"set_option", "value", b"value"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["value", b"value"]) -> typing_extensions.Literal["exception", "echo", "flush", "info", "set_option", "init_chain", "query", "begin_block", "check_tx", "deliver_tx", "end_block", "commit", "list_snapshots", "offer_snapshot", "load_snapshot_chunk", "apply_snapshot_chunk"] | None: ...

global___Response = Response

@typing_extensions.final
class ResponseException(google.protobuf.message.Message):
    """nondeterministic"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ERROR_FIELD_NUMBER: builtins.int
    error: builtins.str
    def __init__(
        self,
        *,
        error: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["error", b"error"]) -> None: ...

global___ResponseException = ResponseException

@typing_extensions.final
class ResponseEcho(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MESSAGE_FIELD_NUMBER: builtins.int
    message: builtins.str
    def __init__(
        self,
        *,
        message: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["message", b"message"]) -> None: ...

global___ResponseEcho = ResponseEcho

@typing_extensions.final
class ResponseFlush(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ResponseFlush = ResponseFlush

@typing_extensions.final
class ResponseInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    APP_VERSION_FIELD_NUMBER: builtins.int
    LAST_BLOCK_HEIGHT_FIELD_NUMBER: builtins.int
    LAST_BLOCK_APP_HASH_FIELD_NUMBER: builtins.int
    data: builtins.str
    version: builtins.str
    app_version: builtins.int
    last_block_height: builtins.int
    last_block_app_hash: builtins.bytes
    def __init__(
        self,
        *,
        data: builtins.str = ...,
        version: builtins.str = ...,
        app_version: builtins.int = ...,
        last_block_height: builtins.int = ...,
        last_block_app_hash: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["app_version", b"app_version", "data", b"data", "last_block_app_hash", b"last_block_app_hash", "last_block_height", b"last_block_height", "version", b"version"]) -> None: ...

global___ResponseInfo = ResponseInfo

@typing_extensions.final
class ResponseSetOption(google.protobuf.message.Message):
    """nondeterministic"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CODE_FIELD_NUMBER: builtins.int
    LOG_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    code: builtins.int
    log: builtins.str
    """bytes data = 2;"""
    info: builtins.str
    def __init__(
        self,
        *,
        code: builtins.int = ...,
        log: builtins.str = ...,
        info: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["code", b"code", "info", b"info", "log", b"log"]) -> None: ...

global___ResponseSetOption = ResponseSetOption

@typing_extensions.final
class ResponseInitChain(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONSENSUS_PARAMS_FIELD_NUMBER: builtins.int
    VALIDATORS_FIELD_NUMBER: builtins.int
    APP_HASH_FIELD_NUMBER: builtins.int
    @property
    def consensus_params(self) -> global___ConsensusParams: ...
    @property
    def validators(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ValidatorUpdate]: ...
    app_hash: builtins.bytes
    def __init__(
        self,
        *,
        consensus_params: global___ConsensusParams | None = ...,
        validators: collections.abc.Iterable[global___ValidatorUpdate] | None = ...,
        app_hash: builtins.bytes = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["consensus_params", b"consensus_params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["app_hash", b"app_hash", "consensus_params", b"consensus_params", "validators", b"validators"]) -> None: ...

global___ResponseInitChain = ResponseInitChain

@typing_extensions.final
class ResponseQuery(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CODE_FIELD_NUMBER: builtins.int
    LOG_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    INDEX_FIELD_NUMBER: builtins.int
    KEY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    PROOF_OPS_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    CODESPACE_FIELD_NUMBER: builtins.int
    code: builtins.int
    log: builtins.str
    """bytes data = 2; // use "value" instead.
    nondeterministic
    """
    info: builtins.str
    """nondeterministic"""
    index: builtins.int
    key: builtins.bytes
    value: builtins.bytes
    @property
    def proof_ops(self) -> tendermint.crypto.proof_pb2.ProofOps: ...
    height: builtins.int
    codespace: builtins.str
    def __init__(
        self,
        *,
        code: builtins.int = ...,
        log: builtins.str = ...,
        info: builtins.str = ...,
        index: builtins.int = ...,
        key: builtins.bytes = ...,
        value: builtins.bytes = ...,
        proof_ops: tendermint.crypto.proof_pb2.ProofOps | None = ...,
        height: builtins.int = ...,
        codespace: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["proof_ops", b"proof_ops"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["code", b"code", "codespace", b"codespace", "height", b"height", "index", b"index", "info", b"info", "key", b"key", "log", b"log", "proof_ops", b"proof_ops", "value", b"value"]) -> None: ...

global___ResponseQuery = ResponseQuery

@typing_extensions.final
class ResponseBeginBlock(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EVENTS_FIELD_NUMBER: builtins.int
    @property
    def events(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Event]: ...
    def __init__(
        self,
        *,
        events: collections.abc.Iterable[global___Event] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["events", b"events"]) -> None: ...

global___ResponseBeginBlock = ResponseBeginBlock

@typing_extensions.final
class ResponseCheckTx(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CODE_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    LOG_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    GAS_WANTED_FIELD_NUMBER: builtins.int
    GAS_USED_FIELD_NUMBER: builtins.int
    EVENTS_FIELD_NUMBER: builtins.int
    CODESPACE_FIELD_NUMBER: builtins.int
    code: builtins.int
    data: builtins.bytes
    log: builtins.str
    """nondeterministic"""
    info: builtins.str
    """nondeterministic"""
    gas_wanted: builtins.int
    gas_used: builtins.int
    @property
    def events(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Event]: ...
    codespace: builtins.str
    def __init__(
        self,
        *,
        code: builtins.int = ...,
        data: builtins.bytes = ...,
        log: builtins.str = ...,
        info: builtins.str = ...,
        gas_wanted: builtins.int = ...,
        gas_used: builtins.int = ...,
        events: collections.abc.Iterable[global___Event] | None = ...,
        codespace: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["code", b"code", "codespace", b"codespace", "data", b"data", "events", b"events", "gas_used", b"gas_used", "gas_wanted", b"gas_wanted", "info", b"info", "log", b"log"]) -> None: ...

global___ResponseCheckTx = ResponseCheckTx

@typing_extensions.final
class ResponseDeliverTx(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CODE_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    LOG_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    GAS_WANTED_FIELD_NUMBER: builtins.int
    GAS_USED_FIELD_NUMBER: builtins.int
    EVENTS_FIELD_NUMBER: builtins.int
    CODESPACE_FIELD_NUMBER: builtins.int
    code: builtins.int
    data: builtins.bytes
    log: builtins.str
    """nondeterministic"""
    info: builtins.str
    """nondeterministic"""
    gas_wanted: builtins.int
    gas_used: builtins.int
    @property
    def events(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Event]: ...
    codespace: builtins.str
    def __init__(
        self,
        *,
        code: builtins.int = ...,
        data: builtins.bytes = ...,
        log: builtins.str = ...,
        info: builtins.str = ...,
        gas_wanted: builtins.int = ...,
        gas_used: builtins.int = ...,
        events: collections.abc.Iterable[global___Event] | None = ...,
        codespace: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["code", b"code", "codespace", b"codespace", "data", b"data", "events", b"events", "gas_used", b"gas_used", "gas_wanted", b"gas_wanted", "info", b"info", "log", b"log"]) -> None: ...

global___ResponseDeliverTx = ResponseDeliverTx

@typing_extensions.final
class ResponseEndBlock(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALIDATOR_UPDATES_FIELD_NUMBER: builtins.int
    CONSENSUS_PARAM_UPDATES_FIELD_NUMBER: builtins.int
    EVENTS_FIELD_NUMBER: builtins.int
    @property
    def validator_updates(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ValidatorUpdate]: ...
    @property
    def consensus_param_updates(self) -> global___ConsensusParams: ...
    @property
    def events(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Event]: ...
    def __init__(
        self,
        *,
        validator_updates: collections.abc.Iterable[global___ValidatorUpdate] | None = ...,
        consensus_param_updates: global___ConsensusParams | None = ...,
        events: collections.abc.Iterable[global___Event] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["consensus_param_updates", b"consensus_param_updates"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["consensus_param_updates", b"consensus_param_updates", "events", b"events", "validator_updates", b"validator_updates"]) -> None: ...

global___ResponseEndBlock = ResponseEndBlock

@typing_extensions.final
class ResponseCommit(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_FIELD_NUMBER: builtins.int
    RETAIN_HEIGHT_FIELD_NUMBER: builtins.int
    data: builtins.bytes
    """reserve 1"""
    retain_height: builtins.int
    def __init__(
        self,
        *,
        data: builtins.bytes = ...,
        retain_height: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["data", b"data", "retain_height", b"retain_height"]) -> None: ...

global___ResponseCommit = ResponseCommit

@typing_extensions.final
class ResponseListSnapshots(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SNAPSHOTS_FIELD_NUMBER: builtins.int
    @property
    def snapshots(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Snapshot]: ...
    def __init__(
        self,
        *,
        snapshots: collections.abc.Iterable[global___Snapshot] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["snapshots", b"snapshots"]) -> None: ...

global___ResponseListSnapshots = ResponseListSnapshots

@typing_extensions.final
class ResponseOfferSnapshot(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Result:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _ResultEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ResponseOfferSnapshot._Result.ValueType], builtins.type):  # noqa: F821
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNKNOWN: ResponseOfferSnapshot._Result.ValueType  # 0
        """Unknown result, abort all snapshot restoration"""
        ACCEPT: ResponseOfferSnapshot._Result.ValueType  # 1
        """Snapshot accepted, apply chunks"""
        ABORT: ResponseOfferSnapshot._Result.ValueType  # 2
        """Abort all snapshot restoration"""
        REJECT: ResponseOfferSnapshot._Result.ValueType  # 3
        """Reject this specific snapshot, try others"""
        REJECT_FORMAT: ResponseOfferSnapshot._Result.ValueType  # 4
        """Reject all snapshots of this format, try others"""
        REJECT_SENDER: ResponseOfferSnapshot._Result.ValueType  # 5
        """Reject all snapshots from the sender(s), try others"""

    class Result(_Result, metaclass=_ResultEnumTypeWrapper): ...
    UNKNOWN: ResponseOfferSnapshot.Result.ValueType  # 0
    """Unknown result, abort all snapshot restoration"""
    ACCEPT: ResponseOfferSnapshot.Result.ValueType  # 1
    """Snapshot accepted, apply chunks"""
    ABORT: ResponseOfferSnapshot.Result.ValueType  # 2
    """Abort all snapshot restoration"""
    REJECT: ResponseOfferSnapshot.Result.ValueType  # 3
    """Reject this specific snapshot, try others"""
    REJECT_FORMAT: ResponseOfferSnapshot.Result.ValueType  # 4
    """Reject all snapshots of this format, try others"""
    REJECT_SENDER: ResponseOfferSnapshot.Result.ValueType  # 5
    """Reject all snapshots from the sender(s), try others"""

    RESULT_FIELD_NUMBER: builtins.int
    result: global___ResponseOfferSnapshot.Result.ValueType
    def __init__(
        self,
        *,
        result: global___ResponseOfferSnapshot.Result.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["result", b"result"]) -> None: ...

global___ResponseOfferSnapshot = ResponseOfferSnapshot

@typing_extensions.final
class ResponseLoadSnapshotChunk(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CHUNK_FIELD_NUMBER: builtins.int
    chunk: builtins.bytes
    def __init__(
        self,
        *,
        chunk: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["chunk", b"chunk"]) -> None: ...

global___ResponseLoadSnapshotChunk = ResponseLoadSnapshotChunk

@typing_extensions.final
class ResponseApplySnapshotChunk(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Result:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _ResultEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ResponseApplySnapshotChunk._Result.ValueType], builtins.type):  # noqa: F821
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNKNOWN: ResponseApplySnapshotChunk._Result.ValueType  # 0
        """Unknown result, abort all snapshot restoration"""
        ACCEPT: ResponseApplySnapshotChunk._Result.ValueType  # 1
        """Chunk successfully accepted"""
        ABORT: ResponseApplySnapshotChunk._Result.ValueType  # 2
        """Abort all snapshot restoration"""
        RETRY: ResponseApplySnapshotChunk._Result.ValueType  # 3
        """Retry chunk (combine with refetch and reject)"""
        RETRY_SNAPSHOT: ResponseApplySnapshotChunk._Result.ValueType  # 4
        """Retry snapshot (combine with refetch and reject)"""
        REJECT_SNAPSHOT: ResponseApplySnapshotChunk._Result.ValueType  # 5
        """Reject this snapshot, try others"""

    class Result(_Result, metaclass=_ResultEnumTypeWrapper): ...
    UNKNOWN: ResponseApplySnapshotChunk.Result.ValueType  # 0
    """Unknown result, abort all snapshot restoration"""
    ACCEPT: ResponseApplySnapshotChunk.Result.ValueType  # 1
    """Chunk successfully accepted"""
    ABORT: ResponseApplySnapshotChunk.Result.ValueType  # 2
    """Abort all snapshot restoration"""
    RETRY: ResponseApplySnapshotChunk.Result.ValueType  # 3
    """Retry chunk (combine with refetch and reject)"""
    RETRY_SNAPSHOT: ResponseApplySnapshotChunk.Result.ValueType  # 4
    """Retry snapshot (combine with refetch and reject)"""
    REJECT_SNAPSHOT: ResponseApplySnapshotChunk.Result.ValueType  # 5
    """Reject this snapshot, try others"""

    RESULT_FIELD_NUMBER: builtins.int
    REFETCH_CHUNKS_FIELD_NUMBER: builtins.int
    REJECT_SENDERS_FIELD_NUMBER: builtins.int
    result: global___ResponseApplySnapshotChunk.Result.ValueType
    @property
    def refetch_chunks(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """Chunks to refetch and reapply"""
    @property
    def reject_senders(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Chunk senders to reject and ban"""
    def __init__(
        self,
        *,
        result: global___ResponseApplySnapshotChunk.Result.ValueType = ...,
        refetch_chunks: collections.abc.Iterable[builtins.int] | None = ...,
        reject_senders: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["refetch_chunks", b"refetch_chunks", "reject_senders", b"reject_senders", "result", b"result"]) -> None: ...

global___ResponseApplySnapshotChunk = ResponseApplySnapshotChunk

@typing_extensions.final
class ConsensusParams(google.protobuf.message.Message):
    """----------------------------------------
    Misc.

    ConsensusParams contains all consensus-relevant parameters
    that can be adjusted by the abci app
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BLOCK_FIELD_NUMBER: builtins.int
    EVIDENCE_FIELD_NUMBER: builtins.int
    VALIDATOR_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    @property
    def block(self) -> global___BlockParams: ...
    @property
    def evidence(self) -> tendermint.types.params_pb2.EvidenceParams: ...
    @property
    def validator(self) -> tendermint.types.params_pb2.ValidatorParams: ...
    @property
    def version(self) -> tendermint.types.params_pb2.VersionParams: ...
    def __init__(
        self,
        *,
        block: global___BlockParams | None = ...,
        evidence: tendermint.types.params_pb2.EvidenceParams | None = ...,
        validator: tendermint.types.params_pb2.ValidatorParams | None = ...,
        version: tendermint.types.params_pb2.VersionParams | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["block", b"block", "evidence", b"evidence", "validator", b"validator", "version", b"version"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["block", b"block", "evidence", b"evidence", "validator", b"validator", "version", b"version"]) -> None: ...

global___ConsensusParams = ConsensusParams

@typing_extensions.final
class BlockParams(google.protobuf.message.Message):
    """BlockParams contains limits on the block size."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MAX_BYTES_FIELD_NUMBER: builtins.int
    MAX_GAS_FIELD_NUMBER: builtins.int
    max_bytes: builtins.int
    """Note: must be greater than 0"""
    max_gas: builtins.int
    """Note: must be greater or equal to -1"""
    def __init__(
        self,
        *,
        max_bytes: builtins.int = ...,
        max_gas: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["max_bytes", b"max_bytes", "max_gas", b"max_gas"]) -> None: ...

global___BlockParams = BlockParams

@typing_extensions.final
class LastCommitInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ROUND_FIELD_NUMBER: builtins.int
    VOTES_FIELD_NUMBER: builtins.int
    round: builtins.int
    @property
    def votes(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___VoteInfo]: ...
    def __init__(
        self,
        *,
        round: builtins.int = ...,
        votes: collections.abc.Iterable[global___VoteInfo] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["round", b"round", "votes", b"votes"]) -> None: ...

global___LastCommitInfo = LastCommitInfo

@typing_extensions.final
class Event(google.protobuf.message.Message):
    """Event allows application developers to attach additional information to
    ResponseBeginBlock, ResponseEndBlock, ResponseCheckTx and ResponseDeliverTx.
    Later, transactions may be queried using these events.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TYPE_FIELD_NUMBER: builtins.int
    ATTRIBUTES_FIELD_NUMBER: builtins.int
    type: builtins.str
    @property
    def attributes(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___EventAttribute]: ...
    def __init__(
        self,
        *,
        type: builtins.str = ...,
        attributes: collections.abc.Iterable[global___EventAttribute] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["attributes", b"attributes", "type", b"type"]) -> None: ...

global___Event = Event

@typing_extensions.final
class EventAttribute(google.protobuf.message.Message):
    """EventAttribute is a single key-value pair, associated with an event."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KEY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    INDEX_FIELD_NUMBER: builtins.int
    key: builtins.bytes
    value: builtins.bytes
    index: builtins.bool
    """nondeterministic"""
    def __init__(
        self,
        *,
        key: builtins.bytes = ...,
        value: builtins.bytes = ...,
        index: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["index", b"index", "key", b"key", "value", b"value"]) -> None: ...

global___EventAttribute = EventAttribute

@typing_extensions.final
class TxResult(google.protobuf.message.Message):
    """TxResult contains results of executing the transaction.

    One usage is indexing transaction results.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HEIGHT_FIELD_NUMBER: builtins.int
    INDEX_FIELD_NUMBER: builtins.int
    TX_FIELD_NUMBER: builtins.int
    RESULT_FIELD_NUMBER: builtins.int
    height: builtins.int
    index: builtins.int
    tx: builtins.bytes
    @property
    def result(self) -> global___ResponseDeliverTx: ...
    def __init__(
        self,
        *,
        height: builtins.int = ...,
        index: builtins.int = ...,
        tx: builtins.bytes = ...,
        result: global___ResponseDeliverTx | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["result", b"result"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["height", b"height", "index", b"index", "result", b"result", "tx", b"tx"]) -> None: ...

global___TxResult = TxResult

@typing_extensions.final
class Validator(google.protobuf.message.Message):
    """----------------------------------------
    Blockchain Types

    Validator
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: builtins.int
    POWER_FIELD_NUMBER: builtins.int
    address: builtins.bytes
    """The first 20 bytes of SHA256(public key)"""
    power: builtins.int
    """PubKey pub_key = 2 [(gogoproto.nullable)=false];
    The voting power
    """
    def __init__(
        self,
        *,
        address: builtins.bytes = ...,
        power: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["address", b"address", "power", b"power"]) -> None: ...

global___Validator = Validator

@typing_extensions.final
class ValidatorUpdate(google.protobuf.message.Message):
    """ValidatorUpdate"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PUB_KEY_FIELD_NUMBER: builtins.int
    POWER_FIELD_NUMBER: builtins.int
    @property
    def pub_key(self) -> tendermint.crypto.keys_pb2.PublicKey: ...
    power: builtins.int
    def __init__(
        self,
        *,
        pub_key: tendermint.crypto.keys_pb2.PublicKey | None = ...,
        power: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pub_key", b"pub_key"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["power", b"power", "pub_key", b"pub_key"]) -> None: ...

global___ValidatorUpdate = ValidatorUpdate

@typing_extensions.final
class VoteInfo(google.protobuf.message.Message):
    """VoteInfo"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALIDATOR_FIELD_NUMBER: builtins.int
    SIGNED_LAST_BLOCK_FIELD_NUMBER: builtins.int
    @property
    def validator(self) -> global___Validator: ...
    signed_last_block: builtins.bool
    def __init__(
        self,
        *,
        validator: global___Validator | None = ...,
        signed_last_block: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["validator", b"validator"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["signed_last_block", b"signed_last_block", "validator", b"validator"]) -> None: ...

global___VoteInfo = VoteInfo

@typing_extensions.final
class Evidence(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TYPE_FIELD_NUMBER: builtins.int
    VALIDATOR_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    TIME_FIELD_NUMBER: builtins.int
    TOTAL_VOTING_POWER_FIELD_NUMBER: builtins.int
    type: global___EvidenceType.ValueType
    @property
    def validator(self) -> global___Validator:
        """The offending validator"""
    height: builtins.int
    """The height when the offense occurred"""
    @property
    def time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """The corresponding time where the offense occurred"""
    total_voting_power: builtins.int
    """Total voting power of the validator set in case the ABCI application does
    not store historical validators.
    https://github.com/tendermint/tendermint/issues/4581
    """
    def __init__(
        self,
        *,
        type: global___EvidenceType.ValueType = ...,
        validator: global___Validator | None = ...,
        height: builtins.int = ...,
        time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        total_voting_power: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["time", b"time", "validator", b"validator"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["height", b"height", "time", b"time", "total_voting_power", b"total_voting_power", "type", b"type", "validator", b"validator"]) -> None: ...

global___Evidence = Evidence

@typing_extensions.final
class Snapshot(google.protobuf.message.Message):
    """----------------------------------------
    State Sync Types
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HEIGHT_FIELD_NUMBER: builtins.int
    FORMAT_FIELD_NUMBER: builtins.int
    CHUNKS_FIELD_NUMBER: builtins.int
    HASH_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    height: builtins.int
    """The height at which the snapshot was taken"""
    format: builtins.int
    """The application-specific snapshot format"""
    chunks: builtins.int
    """Number of chunks in the snapshot"""
    hash: builtins.bytes
    """Arbitrary snapshot hash, equal only if identical"""
    metadata: builtins.bytes
    """Arbitrary application metadata"""
    def __init__(
        self,
        *,
        height: builtins.int = ...,
        format: builtins.int = ...,
        chunks: builtins.int = ...,
        hash: builtins.bytes = ...,
        metadata: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["chunks", b"chunks", "format", b"format", "hash", b"hash", "height", b"height", "metadata", b"metadata"]) -> None: ...

global___Snapshot = Snapshot



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: osmosis/txfees/v1beta1/gov.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from osmosis.txfees.v1beta1 import feetoken_pb2 as osmosis_dot_txfees_dot_v1beta1_dot_feetoken__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n osmosis/txfees/v1beta1/gov.proto\x12\x16osmosis.txfees.v1beta1\x1a\x14gogoproto/gogo.proto\x1a%osmosis/txfees/v1beta1/feetoken.proto\"\xc2\x01\n\x16UpdateFeeTokenProposal\x12\x1f\n\x05title\x18\x01 \x01(\tB\x10\xf2\xde\x1f\x0cyaml:\"title\"\x12+\n\x0b\x64\x65scription\x18\x02 \x01(\tB\x16\xf2\xde\x1f\x12yaml:\"description\"\x12L\n\x08\x66\x65\x65token\x18\x03 \x01(\x0b\x32 .osmosis.txfees.v1beta1.FeeTokenB\x18\xf2\xde\x1f\x10yaml:\"fee_token\"\xc8\xde\x1f\x00:\x0c\xe8\xa0\x1f\x01\x88\xa0\x1f\x00\x98\xa0\x1f\x00\x42\x34Z2github.com/osmosis-labs/osmosis/v14/x/txfees/typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'osmosis.txfees.v1beta1.gov_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z2github.com/osmosis-labs/osmosis/v14/x/txfees/types'
  _UPDATEFEETOKENPROPOSAL.fields_by_name['title']._options = None
  _UPDATEFEETOKENPROPOSAL.fields_by_name['title']._serialized_options = b'\362\336\037\014yaml:\"title\"'
  _UPDATEFEETOKENPROPOSAL.fields_by_name['description']._options = None
  _UPDATEFEETOKENPROPOSAL.fields_by_name['description']._serialized_options = b'\362\336\037\022yaml:\"description\"'
  _UPDATEFEETOKENPROPOSAL.fields_by_name['feetoken']._options = None
  _UPDATEFEETOKENPROPOSAL.fields_by_name['feetoken']._serialized_options = b'\362\336\037\020yaml:\"fee_token\"\310\336\037\000'
  _UPDATEFEETOKENPROPOSAL._options = None
  _UPDATEFEETOKENPROPOSAL._serialized_options = b'\350\240\037\001\210\240\037\000\230\240\037\000'
  _UPDATEFEETOKENPROPOSAL._serialized_start=122
  _UPDATEFEETOKENPROPOSAL._serialized_end=316
# @@protoc_insertion_point(module_scope)
